@@ -9,7 +9,8 @@
 # option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
 # the full text of the license.
 
-import os, glob, subprocess, os.path, time, pwd, sys, stat
+import os, glob, subprocess, os.path, time, pwd, sys, requests_unixsocket
+import stat
 from operator import itemgetter
 
 try:
@@ -109,6 +110,21 @@ def find_file_package(file):
     return packaging.get_file_package(file)
 
 
+def find_snap(snap):
+    '''Return the data of the given snap.
+
+    Return None if the snap is not found to be installed.
+    '''
+    session = requests_unixsocket.Session()
+    try:
+        r = session.get('http+unix://%2Frun%2Fsnapd.socket/v2/snaps/{}'.format(snap))
+        if r.status_code == 200:
+            j = r.json()
+            return j["result"]
+    except Exception:
+        return None
+
+
 def seen_report(report):
     '''Check whether the report file has already been processed earlier.'''
 
@@ -319,9 +335,9 @@ def check_files_md5(sumfile):
         return []
 
     mismatches = []
-    for l in out.splitlines():
-        if l.endswith('FAILED'):
-            mismatches.append(l.rsplit(':', 1)[0])
+    for line in out.splitlines():
+        if line.endswith('FAILED'):
+            mismatches.append(line.rsplit(':', 1)[0])
 
     return mismatches
 
@@ -459,7 +475,7 @@ def find_core_files_by_uid(uid):
             if f.split('.')[2] == str(uid):
                 time = os.path.getmtime(os.path.join(core_dir, f))
                 uid_files.append([f, time])
-        except (IndexError, OSError):
+        except (IndexError, FileNotFoundError):
             continue
     return uid_files
 
