@@ -24,6 +24,7 @@ import base64
 import tempfile
 import shutil
 import locale
+import json
 
 from apport.packaging_impl import impl as packaging
 
@@ -233,7 +234,10 @@ def attach_hardware(report):
     - /proc/cmdline
     - /proc/modules
     - lspci -vvnn
+    - lscpi -vt
     - lsusb
+    - lsusb -v
+    - lsusb -t
     - devices from udev
     - DMI information from /sys
     - prtconf (sparc)
@@ -247,7 +251,10 @@ def attach_hardware(report):
 
     if os.path.exists('/sys/bus/pci'):
         report['Lspci'] = command_output(['lspci', '-vvnn'])
+        report['Lspci-vt'] = command_output(['lspci', '-vt'])
     report['Lsusb'] = command_output(['lsusb'])
+    report['Lsusb-v'] = command_output(['lsusb', '-v'])
+    report['Lsusb-t'] = command_output(['lsusb', '-t'])
     report['ProcModules'] = command_output(['sort', '/proc/modules'])
     report['UdevDb'] = command_output(['udevadm', 'info', '--export-db'])
 
@@ -382,8 +389,8 @@ def command_output(command, input=None, stderr=subprocess.STDOUT,
     if sp.returncode == 0:
         res = out.strip()
     else:
-        res = (b'Error: command ' + str(command).encode() + b' failed with exit code ' +
-               str(sp.returncode).encode() + b': ' + out)
+        res = (b'Error: command ' + str(command).encode() + b' failed with exit code '
+               + str(sp.returncode).encode() + b': ' + out)
 
     if decode_utf8:
         res = res.decode('UTF-8', errors='replace')
@@ -610,9 +617,9 @@ def attach_gsettings_schema(report, schema):
     env['XDG_CONFIG_HOME'] = '/nonexisting'
     gsettings = subprocess.Popen(['gsettings', 'list-recursively', schema],
                                  env=env, stdout=subprocess.PIPE)
-    for l in gsettings.stdout:
+    for line in gsettings.stdout:
         try:
-            (schema_name, key, value) = l.split(None, 2)
+            (schema_name, key, value) = line.split(None, 2)
             value = value.rstrip()
         except ValueError:
             continue  # invalid line
@@ -620,9 +627,9 @@ def attach_gsettings_schema(report, schema):
 
     gsettings = subprocess.Popen(['gsettings', 'list-recursively', schema],
                                  stdout=subprocess.PIPE)
-    for l in gsettings.stdout:
+    for line in gsettings.stdout:
         try:
-            (schema_name, key, value) = l.split(None, 2)
+            (schema_name, key, value) = line.split(None, 2)
             value = value.rstrip()
         except ValueError:
             continue  # invalid line
@@ -806,8 +813,8 @@ def _get_module_license(module):
             return 'invalid'
     except OSError:
         return None
-    for l in out.splitlines():
-        fields = l.split(':', 1)
+    for line in out.splitlines():
+        fields = line.split(':', 1)
         if len(fields) < 2:
             continue
         if fields[0] == 'license':
@@ -821,7 +828,7 @@ def nonfree_kernel_modules(module_list='/proc/modules'):
 
     try:
         with open(module_list) as f:
-            mods = [l.split()[0] for l in f]
+            mods = [line.split()[0] for line in f]
     except IOError:
         return []
 
@@ -917,12 +924,26 @@ def attach_default_grub(report, key=None):
 
     if os.path.exists(path):
         with open(path, 'r') as f:
-            filtered = [l if not l.startswith('password')
+            filtered = [line if not line.startswith('password')
                         else '### PASSWORD LINE REMOVED ###'
-                        for l in f.readlines()]
+                        for line in f.readlines()]
             report[key] = ''.join(filtered)
 
 
+def attach_casper_md5check(report, location):
+    '''attach the results of the casper md5check of install media'''
+    result = 'skip'
+    mismatches = []
+    if os.path.exists(location):
+        with open(location) as json_file:
+            check = json.load(json_file)
+            result = check['result']
+            mismatches = check['checksum_missmatch']
+    report['CasperMD5CheckResult'] = result
+    if mismatches:
+        report['CasperMD5CheckMismatches'] = ' '.join(mismatches)
+
+
 # backwards compatible API
 shared_libraries = apport.fileutils.shared_libraries
 links_with_shared_library = apport.fileutils.links_with_shared_library
