@@ -13,7 +13,6 @@ This is used on Debian and derivatives such as Ubuntu.
 # the full text of the license.
 
 import subprocess, os, glob, stat, sys, tempfile, shutil, time
-import errno
 import hashlib
 import json
 
@@ -50,8 +49,10 @@ class __AptDpkgPackageInfo(PackageInfo):
         self._contents_dir = None
         self._mirror = None
         self._virtual_mapping_obj = None
+        self._contents_mapping_obj = None
         self._launchpad_base = 'https://api.launchpad.net/devel'
         self._ppa_archive_url = self._launchpad_base + '/~%(user)s/+archive/%(distro)s/%(ppaname)s'
+        self._contents_update = False
 
     def __del__(self):
         try:
@@ -79,6 +80,36 @@ class __AptDpkgPackageInfo(PackageInfo):
             with open(mapping_file, 'wb') as fp:
                 pickle.dump(self._virtual_mapping_obj, fp)
 
+    def _contents_mapping(self, configdir, release, arch):
+        if self._contents_mapping_obj and \
+                self._contents_mapping_obj['release'] == release and \
+                self._contents_mapping_obj['arch'] == arch:
+            return self._contents_mapping_obj
+
+        mapping_file = os.path.join(configdir, 'contents_mapping-%s-%s.pickle' %
+                                    (release, arch))
+        if os.path.exists(mapping_file) and os.stat(mapping_file).st_size == 0:
+            os.remove(mapping_file)
+        if os.path.exists(mapping_file):
+            with open(mapping_file, 'rb') as fp:
+                self._contents_mapping_obj = pickle.load(fp)
+        else:
+            self._contents_mapping_obj = {'release': release, 'arch': arch}
+
+        return self._contents_mapping_obj
+
+    def _save_contents_mapping(self, configdir, release, arch):
+        mapping_file = os.path.join(configdir, 'contents_mapping-%s-%s.pickle' %
+                                    (release, arch))
+        if self._contents_mapping_obj is not None:
+            try:
+                with open(mapping_file, 'wb') as fp:
+                    pickle.dump(self._contents_mapping_obj, fp)
+            # rather than crashing on systems with little memory just don't
+            # write the crash file
+            except MemoryError:
+                pass
+
     def _cache(self):
         '''Return apt.Cache() (initialized lazily).'''
 
@@ -232,6 +263,8 @@ class __AptDpkgPackageInfo(PackageInfo):
         return False
 
     def get_lp_binary_package(self, distro_id, release, package, version, arch):
+        # allow unauthenticated downloads
+        apt.apt_pkg.config.set('APT::Get::AllowUnauthenticated', 'True')
         from launchpadlib.launchpad import Launchpad
         launchpad = Launchpad.login_anonymously('apport-retrace', 'production',
                                                 version='devel')
@@ -299,6 +332,7 @@ class __AptDpkgPackageInfo(PackageInfo):
                                           exact_match=True)
         if not pss:
             return None
+        sfus = ''
         for ps in pss:
             if ps.status == 'Deleted':
                 continue
@@ -390,7 +424,7 @@ class __AptDpkgPackageInfo(PackageInfo):
         allows filtering.
         '''
         dpkg = subprocess.Popen(['dpkg-query', '-W', '--showformat=${Conffiles}',
-                                 package], stdout=subprocess.PIPE)
+                                 '--', package], stdout=subprocess.PIPE)
 
         out = dpkg.communicate()[0].decode()
         if dpkg.returncode != 0:
@@ -954,6 +988,10 @@ Debug::NoLocking "true";
                                                                              destdir=archivedir))
                                 lp_cache[dbg_pkg] = ver
                                 pkg_found = True
+                            # if it can't be found in Launchpad failover to a
+                            # code path that'll use -dbgsym packages
+                            else:
+                                raise KeyError
                     if not pkg_found:
                         try:
                             dbg.candidate = dbg.versions[candidate.version]
@@ -971,6 +1009,12 @@ Debug::NoLocking "true";
                         dbgs = [p for p in src_records.binaries
                                 if p.endswith('-dbg') and p in cache and
                                 'transitional' not in cache[p].candidate.description]
+                        # if a specific version of a package was requested
+                        # only install dbg pkgs whose version matches
+                        if ver:
+                            for dbg in dbgs:
+                                if cache[dbg].candidate.version != ver:
+                                    dbgs.remove(dbg)
                     else:
                         dbgs = []
                     if dbgs:
@@ -1175,9 +1219,9 @@ Debug::NoLocking "true";
             return []
 
         mismatches = []
-        for l in out.splitlines():
-            if l.endswith('FAILED'):
-                mismatches.append(l.rsplit(':', 1)[0])
+        for line in out.splitlines():
+            if line.endswith('FAILED'):
+                mismatches.append(line.rsplit(':', 1)[0])
 
         return mismatches
 
@@ -1186,15 +1230,16 @@ Debug::NoLocking "true";
         '''Heuristically determine primary mirror from an apt sources.list'''
 
         with open(apt_sources) as f:
-            for l in f:
-                fields = l.split()
+            for line in f:
+                fields = line.split()
                 if len(fields) >= 3 and fields[0] == 'deb':
                     if fields[1].startswith('['):
                         # options given, mirror is in third field
                         mirror_idx = 2
                     else:
                         mirror_idx = 1
-                    if fields[mirror_idx].startswith('http://'):
+                    if fields[mirror_idx].startswith('http://') or \
+                            fields[mirror_idx].startswith('https://'):
                         return fields[mirror_idx]
             else:
                 raise SystemError('cannot determine default mirror: %s does not contain a valid deb line'
@@ -1236,12 +1281,14 @@ Debug::NoLocking "true";
             release = self.get_distro_codename()
         else:
             release = self._distro_release_to_codename(release)
-        # search -proposed last since file may be provided by a new package
-        for pocket in ['-updates', '-security', '', '-proposed']:
-            map = os.path.join(dir, '%s%s-Contents-%s.gz' % (release, pocket, arch))
-
-            # check if map exists and is younger than a day; if not, we need to
-            # refresh it
+        # this is ordered by likelihood of installation with the most common
+        # last
+        for pocket in ['-proposed', '', '-security', '-updates']:
+            map = os.path.join(dir, '%s%s-Contents-%s.gz' %
+                                    (release, pocket, arch))
+            # check if map exists and is younger than a day; if not, we need
+            # to refresh it
+            update = False
             try:
                 st = os.stat(map)
                 age = int(time.time() - st.st_mtime)
@@ -1275,6 +1322,7 @@ Debug::NoLocking "true";
                 else:
                     update = True
                 if update:
+                    self._contents_update = True
                     try:
                         src = urlopen(url)
                     except IOError:
@@ -1294,32 +1342,73 @@ Debug::NoLocking "true";
                     src.close()
                     assert os.path.exists(map)
 
-            if file.startswith('/'):
-                file = file[1:]
-
-            # zgrep is magnitudes faster than a 'gzip.open/split() loop'
-            package = None
-            try:
-                zgrep = subprocess.Popen(['zgrep', '-m1', '^%s[[:space:]]' % file, map],
-                                         stdout=subprocess.PIPE, stderr=subprocess.PIPE)
-                out = zgrep.communicate()[0].decode('UTF-8')
-            except OSError as e:
-                if e.errno != errno.ENOMEM:
-                    raise
-                file_b = file.encode()
+            contents_mapping = self._contents_mapping(dir, release, arch)
+            # if the mapping is empty build it
+            if not contents_mapping or len(contents_mapping) == 2:
+                self._contents_update = True
+            # if any of the Contents files were updated we need to update the
+            # map because the ordering in which is created is important
+            if self._contents_update:
                 import gzip
                 with gzip.open('%s' % map, 'rb') as contents:
-                    out = ''
+                    line_num = 0
                     for line in contents:
-                        if line.startswith(file_b):
-                            out = line
-                            break
-            # we do not check the return code, since zgrep -m1 often errors out
-            # with 'stdout: broken pipe'
-            if out:
-                package = out.split()[1].split(',')[0].split('/')[-1]
-            if package:
-                return package
+                        line_num += 1
+                        # the first 32 lines are descriptive only for these
+                        # releases
+                        if pocket == '' and release in ['trusty', 'xenial'] \
+                                and line_num < 33:
+                            continue
+                        path = line.split()[0]
+                        if path.split(b'/')[0] == b'usr':
+                            if path.split(b'/')[1] not in (b'lib', b'libexec', b'libx32', b'bin', b'sbin', b'share', b'games', b'Brother'):
+                                continue
+                            if path.split(b'/')[1] == b'share' and \
+                                    path.split(b'/')[2] in (b'doc', b'icons', b'man', b'texlive', b'gocode', b'locale', b'help'):
+                                continue
+                            package = line.split()[-1].split(b',')[0].split(b'/')[-1]
+                        elif path.split(b'/')[0] in (b'lib', b'bin', b'sbin'):
+                            package = line.split()[-1].split(b',')[0].split(b'/')[-1]
+                        else:
+                            continue
+                        if path in contents_mapping:
+                            if package == contents_mapping[path]:
+                                continue
+                            else:
+                                # if the package was updated use the update
+                                # b/c everyone should have packages from
+                                # -updates and -security installed
+                                contents_mapping[path] = package
+                        else:
+                            contents_mapping[path] = package
+        # the file only needs to be saved after an update
+        if self._contents_update:
+            self._save_contents_mapping(dir, release, arch)
+            # the update of the mapping only needs to be done once
+            self._contents_update = False
+        if isinstance(file, bytes):
+            file = file.decode()
+        if file[0] == '/':
+            file = file[1:]
+        files = [file.encode()]
+        if file.startswith('usr/lib/x86_64-linux-gnu/'):
+            files.append(file[4:].encode())
+        elif file.startswith('usr/lib/i386-linux-gnu/'):
+            files.append(file[4:].encode())
+        elif file.startswith('usr/lib/systemd/'):
+            files.append(file[4:].encode())
+        elif file.startswith('usr/lib/udev/'):
+            files.append(file[4:].encode())
+        elif file.startswith('usr/bin/'):
+            files.append(file[4:].encode())
+        elif file.startswith('usr/sbin/'):
+            files.append(file[4:].encode())
+        for file in files:
+            try:
+                pkg = contents_mapping[file].decode()
+                return pkg
+            except KeyError:
+                pass
         return None
 
     @classmethod
