@@ -19,10 +19,12 @@ if sys.version > '3':
     _python2 = False
     from urllib.error import URLError
     from urllib.request import urlopen
+    from urllib.parse import unquote
     (urlopen, URLError)  # pyflakes
 else:
     _python2 = True
     from urllib import urlopen
+    from urllib import unquote
     URLError = IOError
 
 import problem_report
@@ -119,8 +121,15 @@ def _command_output(command, input=None, env=None):
     '''
     sp = subprocess.Popen(command, stdout=subprocess.PIPE,
                           stderr=subprocess.STDOUT, env=env)
-
-    out = sp.communicate(input)[0]
+    # gdb can timeout when trying to retrace some core files, giving it 30
+    # minutes to run should be more than enough.
+    try:
+        out = sp.communicate(input=input, timeout=1800)[0]
+    except subprocess.TimeoutExpired:
+        sp.kill()
+        out, errs = sp.communicate()
+        raise OSError('Error: command %s timedout with exit code %i: %s' % (
+            str(command), sp.returncode, out))
     if sp.returncode == 0:
         return out
     else:
@@ -225,7 +234,7 @@ def _run_hook(report, ui, hook):
                 raise
     except StopIteration:
         return True
-    except Exception as e:
+    except Exception:
         hookname = os.path.splitext(os.path.basename(hook))[0].replace('-', '_')
         report['HookError_' + hookname] = traceback.format_exc()
         apport.error('hook %s crashed:', hook)
@@ -344,7 +353,7 @@ class Report(problem_report.ProblemReport):
             try:
                 self['SourcePackage'] = packaging.get_source(package)
             except ValueError:
-                # might not exist for non-free third-party packages
+                # might not exist for non-free third-party packages or snaps
                 pass
         if not version:
             return
@@ -369,6 +378,26 @@ class Report(problem_report.ProblemReport):
             self['Dependencies'] += '%s %s%s' % (
                 dep, v, self._customized_package_suffix(dep))
 
+    def add_snap_info(self, snap):
+        '''Add info about an installed Snap
+
+        This adds a Snap: field, containing name, version and channel.
+        It adds a SnapSource: field, if the snap has a Launchpad contact defined.
+        '''
+        self['Snap'] = '%s %s (%s)' % (snap.get("name"), snap.get("version"),
+                                       snap.get("channel", "unknown"))
+        # Automatically handle snaps which have a Launchpad contact defined
+        if snap.get("contact"):
+            # Parse Launchpad project (e.g. 'subiquity') or source package string
+            # (e.g. 'ubuntu/+source/gnome-calculator') from snap 'contact'
+            # Additionaly, extract any tag/tags defined in the contact URL.
+            p = r'^https?:\/\/.*launchpad\.net\/((?:[^\/]+\/\+source\/)?[^\/]+)(?:.*field\.tags?=([^&]+))?'
+            m = re.search(p, unquote(snap.get("contact", "")))
+            if m and m.group(1):
+                self['SnapSource'] = m.group(1)
+                if m.group(2):
+                    self['SnapTags'] = m.group(2)
+
     def add_os_info(self):
         '''Add operating system information.
 
@@ -396,7 +425,12 @@ class Report(problem_report.ProblemReport):
         groups = [name for name, p, gid, memb in grp.getgrall()
                   if user in memb and gid < 1000]
         groups.sort()
-        self['UserGroups'] = ' '.join(groups)
+        if groups:
+            self['UserGroups'] = ' '.join(groups)
+        else:
+            # the docs indicate this is optional but a lot of tests expect
+            # UserGroups to exist
+            self['UserGroups'] = 'N/A'
 
     def _check_interpreted(self):
         '''Check if process is a script.
@@ -430,7 +464,7 @@ class Report(problem_report.ProblemReport):
             return
 
         cmdargs = self['ProcCmdline'].split('\0')
-        bindirs = ['/bin/', '/sbin/', '/usr/bin/', '/usr/sbin/']
+        bindirs = ['/usr/bin/', '/usr/sbin/', '/bin/', '/sbin/']
 
         # filter out interpreter options
         while len(cmdargs) >= 2 and cmdargs[1].startswith('-'):
@@ -518,7 +552,7 @@ class Report(problem_report.ProblemReport):
             pathlist = [path]
 
             if not module and desc[2] == imp.PKG_DIRECTORY:
-                    module = ['__init__']
+                module = ['__init__']
 
         if path and path.endswith('.pyc'):
             path = path[:-1]
@@ -577,6 +611,8 @@ class Report(problem_report.ProblemReport):
             try:
                 self['ExecutablePath'] = _read_proc_link('exe', pid, proc_pid_fd)
             except OSError as e:
+                if e.errno in (errno.EPERM, errno.EACCES):
+                    raise ValueError('not accessible')
                 if e.errno == errno.ENOENT:
                     raise ValueError('invalid process')
                 else:
@@ -732,7 +768,8 @@ class Report(problem_report.ProblemReport):
         files there.
 
         Raises a IOError if the core dump is invalid/truncated, or OSError if
-        calling gdb fails.
+        calling gdb fails, or FileNotFoundError if gdb or the crashing
+        executable cannot be found.
         '''
         if 'CoreDump' not in self or 'ExecutablePath' not in self:
             return
@@ -745,6 +782,9 @@ class Report(problem_report.ProblemReport):
                        'GLibAssertionMessage': 'print __glib_assert_msg',
                        'NihAssertionMessage': 'print (char*) __nih_abort_msg'}
         gdb_cmd, environ = self.gdb_command(rootdir, gdb_sandbox)
+        if not gdb_cmd:
+            raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT),
+                                    'gdb not found in retracing env')
 
         # limit maximum backtrace depth (to avoid looped stacks)
         gdb_cmd += ['--batch', '--ex', 'set backtrace limit 2000']
@@ -754,18 +794,27 @@ class Report(problem_report.ProblemReport):
         for name, cmd in gdb_reports.items():
             value_keys.append(name)
             gdb_cmd += ['--ex', 'p -99', '--ex', cmd]
+        # End with our separator, ensures gdb's return code is as expected
+        value_keys.append('separator')
+        gdb_cmd += ['--ex', 'p -99']
 
         out = _command_output(gdb_cmd, env=environ).decode('UTF-8', errors='replace')
 
         # check for truncated stack trace
-        if 'is truncated: expected core file size' in out:
+        if 'is truncated: expected core file size' in out or \
+                'is not a core dump: file truncated' in out:
             if 'warning:' in out:
                 warnings = '\n'.join([l for l in out.splitlines() if 'warning:' in l])
             elif 'Warning:' in out:
                 warnings = '\n'.join([l for l in out.splitlines() if 'Warning:' in l])
+            else:
+                warnings = out.splitlines()[0]
             reason = 'Invalid core dump: ' + warnings.strip()
             self['UnreportableReason'] = reason
             raise IOError(reason)
+        elif out.split('\n')[0].endswith('No such file or directory.'):
+            raise FileNotFoundError(errno.ENOENT, os.strerror(errno.ENOENT),
+                                    'executable file for coredump not found')
 
         # split the output into the various fields
         part_re = re.compile(r'^\$\d+\s*=\s*-99$', re.MULTILINE)
@@ -1472,6 +1521,8 @@ class Report(problem_report.ProblemReport):
         '''
         if 'ProcMaps' not in self or 'Stacktrace' not in self or 'Signal' not in self:
             return None
+        if 'Errno 13' in self['ProcMaps']:
+            return None
 
         stack = []
         failed = 0
@@ -1578,8 +1629,7 @@ class Report(problem_report.ProblemReport):
                            if gdb_sandbox else None)
         gdb_path = _which_extrapath('gdb', gdb_sandbox_bin)
         if not gdb_path:
-            apport.fatal('gdb does not exist in the %ssandbox nor on the host'
-                         % ('gdb ' if not same_arch else ''))
+            return '', ''
         command = [gdb_path]
         environ = None
 
@@ -1603,8 +1653,8 @@ class Report(problem_report.ProblemReport):
                         '--ex', 'add-auto-load-safe-path ' + sandbox,
                         # needed to fix /lib64/ld-linux-x86-64.so.2 broken
                         # symlink
-                        '--ex', 'set solib-search-path %s/lib/%s' % \
-                        (sandbox, native_multiarch)]
+                        '--ex', 'set solib-search-path %s/lib/%s:%s/usr/lib/%s' % \
+                        (sandbox, native_multiarch, sandbox, native_multiarch)]
             if gdb_sandbox:
                 ld_lib_path = '%s/lib:%s/lib/%s:%s/usr/lib/%s:%s/usr/lib' % \
                     (gdb_sandbox, gdb_sandbox, native_multiarch,
@@ -1621,7 +1671,6 @@ class Report(problem_report.ProblemReport):
                             gdb_sandbox]
             executable = sandbox + '/' + executable
 
-        assert os.path.exists(executable)
         command += ['--ex', 'file "%s"' % executable]
 
         if 'CoreDump' in self:
@@ -1706,7 +1755,7 @@ class Report(problem_report.ProblemReport):
         session, or None otherwise.
         '''
         if not _python2 and proc_pid_fd is not None:
-            cgroup_file = os.open('cgroup', os.O_RDONLY | os.O_CLOEXEC, dir_fd=proc_pid_fd)
+            cgroup_file = os.open('cgroup', os.O_RDONLY, dir_fd=proc_pid_fd)
         else:
             cgroup_file = "/proc/%s/cgroup" % pid
 
