@@ -239,12 +239,6 @@ def purge_cache_on_python_version_change(init):
         util.write_file(python_version_path, current_python_version)
 
 
-def _should_bring_up_interfaces(init, args):
-    if util.get_cfg_option_bool(init.cfg, 'disable_network_activation'):
-        return False
-    return not args.local
-
-
 def main_init(name, args):
     deps = [sources.DEP_FILESYSTEM, sources.DEP_NETWORK]
     if args.local:
@@ -354,7 +348,6 @@ def main_init(name, args):
         util.del_file(os.path.join(path_helper.get_cpath("data"), "no-net"))
 
     # Stage 5
-    bring_up_interfaces = _should_bring_up_interfaces(init, args)
     try:
         init.fetch(existing=existing)
         # if in network mode, and the datasource is local
@@ -374,7 +367,7 @@ def main_init(name, args):
             util.logexc(LOG, ("No instance datasource found!"
                               " Likely bad things to come!"))
         if not args.force:
-            init.apply_network_config(bring_up=bring_up_interfaces)
+            init.apply_network_config(bring_up=not args.local)
             LOG.debug("[%s] Exiting without datasource", mode)
             if mode == sources.DSMODE_LOCAL:
                 return (None, [])
@@ -395,7 +388,7 @@ def main_init(name, args):
         # dhcp clients to advertize this hostname to any DDNS services
         # LP: #1746455.
         _maybe_set_hostname(init, stage='local', retry_stage='network')
-    init.apply_network_config(bring_up=bring_up_interfaces)
+    init.apply_network_config(bring_up=bool(mode != sources.DSMODE_LOCAL))
 
     if mode == sources.DSMODE_LOCAL:
         if init.datasource.dsmode != mode:
