@@ -65,19 +65,15 @@ swap file is created.
 from string import whitespace
 
 import logging
-import os
+import os.path
 import re
 
 from cloudinit import type_utils
-from cloudinit import subp
 from cloudinit import util
 
 # Shortname matches 'sda', 'sda1', 'xvda', 'hda', 'sdb', xvdb, vda, vdd1, sr0
 DEVICE_NAME_FILTER = r"^([x]{0,1}[shv]d[a-z][0-9]*|sr[0-9]+)$"
 DEVICE_NAME_RE = re.compile(DEVICE_NAME_FILTER)
-# Name matches 'server:/path'
-NETWORK_NAME_FILTER = r"^.+:.*"
-NETWORK_NAME_RE = re.compile(NETWORK_NAME_FILTER)
 WS = re.compile("[%s]+" % (whitespace))
 FSTAB_PATH = "/etc/fstab"
 MNT_COMMENT = "comment=cloudconfig"
@@ -97,13 +93,6 @@ def is_meta_device_name(name):
     return False
 
 
-def is_network_device(name):
-    # return true if this is a network device
-    if NETWORK_NAME_RE.match(name):
-        return True
-    return False
-
-
 def _get_nth_partition_for_device(device_path, partition_number):
     potential_suffixes = [str(partition_number), 'p%s' % (partition_number,),
                           '-part%s' % (partition_number,)]
@@ -123,7 +112,7 @@ def _is_block_device(device_path, partition_path=None):
     return os.path.exists(sys_path)
 
 
-def sanitize_devname(startname, transformer, log, aliases=None):
+def sanitize_devname(startname, transformer, log):
     log.debug("Attempting to determine the real name of %s", startname)
 
     # workaround, allow user to specify 'ephemeral'
@@ -133,18 +122,10 @@ def sanitize_devname(startname, transformer, log, aliases=None):
         devname = "ephemeral0"
         log.debug("Adjusted mount option from ephemeral to ephemeral0")
 
-    if is_network_device(startname):
-        return startname
-
     device_path, partition_number = util.expand_dotted_devname(devname)
-    orig = device_path
-
-    if aliases:
-        device_path = aliases.get(device_path, device_path)
-        if orig != device_path:
-            log.debug("Mapped device alias %s to %s", orig, device_path)
 
     if is_meta_device_name(device_path):
+        orig = device_path
         device_path = transformer(device_path)
         if not device_path:
             return None
@@ -258,32 +239,30 @@ def create_swapfile(fname: str, size: str) -> None:
                    'count=%s' % size]
 
         try:
-            subp.subp(cmd, capture=True)
-        except subp.ProcessExecutionError as e:
-            LOG.info(errmsg, fname, size, method, e)
+            util.subp(cmd, capture=True)
+        except util.ProcessExecutionError as e:
+            LOG.warning(errmsg, fname, size, method, e)
             util.del_file(fname)
-            raise
 
     swap_dir = os.path.dirname(fname)
     util.ensure_dir(swap_dir)
 
     fstype = util.get_mount_info(swap_dir)[1]
 
-    if (fstype == "xfs" and
-            util.kernel_version() < (4, 18)) or fstype == "btrfs":
+    if fstype in ("xfs", "btrfs"):
         create_swap(fname, size, "dd")
     else:
         try:
             create_swap(fname, size, "fallocate")
-        except subp.ProcessExecutionError:
-            LOG.info("fallocate swap creation failed, will attempt with dd")
+        except util.ProcessExecutionError as e:
+            LOG.warning(errmsg, fname, size, "dd", e)
+            LOG.warning("Will attempt with dd.")
             create_swap(fname, size, "dd")
 
-    if os.path.exists(fname):
-        util.chmod(fname, 0o600)
+    util.chmod(fname, 0o600)
     try:
-        subp.subp(['mkswap', fname])
-    except subp.ProcessExecutionError:
+        util.subp(['mkswap', fname])
+    except util.ProcessExecutionError:
         util.del_file(fname)
         raise
 
@@ -295,6 +274,7 @@ def setup_swapfile(fname, size=None, maxsize=None):
     maxsize: the maximum size
     """
     swap_dir = os.path.dirname(fname)
+    mibsize = str(int(size / (2 ** 20)))
     if str(size).lower() == "auto":
         try:
             memsize = util.read_meminfo()['total']
@@ -306,7 +286,6 @@ def setup_swapfile(fname, size=None, maxsize=None):
         size = suggested_swapsize(fsys=swap_dir, maxsize=maxsize,
                                   memsize=memsize)
 
-    mibsize = str(int(size / (2 ** 20)))
     if not size:
         LOG.debug("Not creating swap: suggested size was 0")
         return
@@ -386,20 +365,17 @@ def handle(_name, cfg, cloud, log, _args):
     fstab_devs = {}
     fstab_removed = []
 
-    if os.path.exists(FSTAB_PATH):
-        for line in util.load_file(FSTAB_PATH).splitlines():
-            if MNT_COMMENT in line:
-                fstab_removed.append(line)
-                continue
+    for line in util.load_file(FSTAB_PATH).splitlines():
+        if MNT_COMMENT in line:
+            fstab_removed.append(line)
+            continue
 
-            try:
-                toks = WS.split(line)
-            except Exception:
-                pass
-            fstab_devs[toks[0]] = line
-            fstab_lines.append(line)
-
-    device_aliases = cfg.get("device_aliases", {})
+        try:
+            toks = WS.split(line)
+        except Exception:
+            pass
+        fstab_devs[toks[0]] = line
+        fstab_lines.append(line)
 
     for i in range(len(cfgmnt)):
         # skip something that wasn't a list
@@ -409,8 +385,7 @@ def handle(_name, cfg, cloud, log, _args):
             continue
 
         start = str(cfgmnt[i][0])
-        sanitized = sanitize_devname(start, cloud.device_name_to_device, log,
-                                     aliases=device_aliases)
+        sanitized = sanitize_devname(start, cloud.device_name_to_device, log)
         if sanitized != start:
             log.debug("changed %s => %s" % (start, sanitized))
 
@@ -452,8 +427,7 @@ def handle(_name, cfg, cloud, log, _args):
     # entry has the same device name
     for defmnt in defmnts:
         start = defmnt[0]
-        sanitized = sanitize_devname(start, cloud.device_name_to_device, log,
-                                     aliases=device_aliases)
+        sanitized = sanitize_devname(start, cloud.device_name_to_device, log)
         if sanitized != start:
             log.debug("changed default device %s => %s" % (start, sanitized))
 
@@ -551,9 +525,9 @@ def handle(_name, cfg, cloud, log, _args):
     for cmd in activate_cmds:
         fmt = "Activate mounts: %s:" + ' '.join(cmd)
         try:
-            subp.subp(cmd)
+            util.subp(cmd)
             log.debug(fmt, "PASS")
-        except subp.ProcessExecutionError:
+        except util.ProcessExecutionError:
             log.warning(fmt, "FAIL")
             util.logexc(log, fmt, "FAIL")
 
