@@ -8,18 +8,15 @@
 
 """Resizefs: cloud-config module which resizes the filesystem"""
 
-
 import errno
-import getopt
 import os
-import re
-import shlex
 import stat
 from textwrap import dedent
 
 from cloudinit.config.schema import (
     get_schema_doc, validate_cloudconfig_schema)
 from cloudinit.settings import PER_ALWAYS
+from cloudinit import subp
 from cloudinit import util
 
 NOBLOCK = "noblock"
@@ -81,65 +78,34 @@ def _resize_xfs(mount_point, devpth):
 
 
 def _resize_ufs(mount_point, devpth):
-    return ('growfs', '-y', devpth)
+    return ('growfs', '-y', mount_point)
 
 
 def _resize_zfs(mount_point, devpth):
     return ('zpool', 'online', '-e', mount_point, devpth)
 
 
-def _get_dumpfs_output(mount_point):
-    return util.subp(['dumpfs', '-m', mount_point])[0]
-
-
-def _get_gpart_output(part):
-    return util.subp(['gpart', 'show', part])[0]
+def _resize_hammer2(mount_point, devpth):
+    return ('hammer2', 'growfs', mount_point)
 
 
 def _can_skip_resize_ufs(mount_point, devpth):
-    # extract the current fs sector size
-    """
-    # dumpfs -m /
-    # newfs command for / (/dev/label/rootfs)
-      newfs -O 2 -U -a 4 -b 32768 -d 32768 -e 4096 -f 4096 -g 16384
-            -h 64 -i 8192 -j -k 6408 -m 8 -o time -s 58719232 /dev/label/rootf
-    """
-    cur_fs_sz = None
-    frag_sz = None
-    dumpfs_res = _get_dumpfs_output(mount_point)
-    for line in dumpfs_res.splitlines():
-        if not line.startswith('#'):
-            newfs_cmd = shlex.split(line)
-            opt_value = 'O:Ua:s:b:d:e:f:g:h:i:jk:m:o:'
-            optlist, _args = getopt.getopt(newfs_cmd[1:], opt_value)
-            for o, a in optlist:
-                if o == "-s":
-                    cur_fs_sz = int(a)
-                if o == "-f":
-                    frag_sz = int(a)
-    # check the current partition size
-    """
-    # gpart show /dev/da0
-=>      40  62914480  da0  GPT  (30G)
-        40      1024    1  freebsd-boot  (512K)
-      1064  58719232    2  freebsd-ufs  (28G)
-  58720296   3145728    3  freebsd-swap  (1.5G)
-  61866024   1048496       - free -  (512M)
-    """
-    expect_sz = None
-    m = re.search('^(/dev/.+)p([0-9])$', devpth)
-    gpart_res = _get_gpart_output(m.group(1))
-    for line in gpart_res.splitlines():
-        if re.search(r"freebsd-ufs", line):
-            fields = line.split()
-            expect_sz = int(fields[1])
-    # Normalize the gpart sector size,
-    # because the size is not exactly the same as fs size.
-    normal_expect_sz = (expect_sz - expect_sz % (frag_sz / 512))
-    if normal_expect_sz == cur_fs_sz:
-        return True
-    else:
-        return False
+    # possible errors cases on the code-path to growfs -N following:
+    # https://github.com/freebsd/freebsd/blob/HEAD/sbin/growfs/growfs.c
+    # This is the "good" error:
+    skip_start = "growfs: requested size"
+    skip_contain = "is not larger than the current filesystem size"
+    # growfs exits with 1 for almost all cases up to this one.
+    # This means we can't just use rcs=[0, 1] as subp parameter:
+    try:
+        subp.subp(['growfs', '-N', devpth])
+    except subp.ProcessExecutionError as e:
+        if e.stderr.startswith(skip_start) and skip_contain in e.stderr:
+            # This FS is already at the desired size
+            return True
+        else:
+            raise e
+    return False
 
 
 # Do not use a dictionary as these commands should be able to be used
@@ -151,6 +117,7 @@ RESIZE_FS_PREFIXES_CMDS = [
     ('xfs', _resize_xfs),
     ('ufs', _resize_ufs),
     ('zfs', _resize_zfs),
+    ('hammer2', _resize_hammer2),
 ]
 
 RESIZE_FS_PRECHECK_CMDS = {
@@ -183,7 +150,7 @@ def maybe_get_writable_device_path(devpath, info, log):
             not container):
         devpath = util.rootdev_from_cmdline(util.get_cmdline())
         if devpath is None:
-            log.warn("Unable to find device '/dev/root'")
+            log.warning("Unable to find device '/dev/root'")
             return None
         log.debug("Converted /dev/root to '%s' per kernel cmdline", devpath)
 
@@ -197,6 +164,13 @@ def maybe_get_writable_device_path(devpath, info, log):
     if devpath.startswith('gpt/'):
         log.debug('We have a gpt label - just go ahead')
         return devpath
+    # Alternatively, our device could simply be a name as returned by gpart,
+    # such as da0p3
+    if not devpath.startswith('/dev/') and not os.path.exists(devpath):
+        fulldevpath = '/dev/' + devpath.lstrip('/')
+        log.debug("'%s' doesn't appear to be a valid device path. Trying '%s'",
+                  devpath, fulldevpath)
+        devpath = fulldevpath
 
     try:
         statret = os.stat(devpath)
@@ -205,8 +179,8 @@ def maybe_get_writable_device_path(devpath, info, log):
             log.debug("Device '%s' did not exist in container. "
                       "cannot resize: %s", devpath, info)
         elif exc.errno == errno.ENOENT:
-            log.warn("Device '%s' did not exist. cannot resize: %s",
-                     devpath, info)
+            log.warning("Device '%s' did not exist. cannot resize: %s",
+                        devpath, info)
         else:
             raise exc
         return None
@@ -216,8 +190,8 @@ def maybe_get_writable_device_path(devpath, info, log):
             log.debug("device '%s' not a block device in container."
                       " cannot resize: %s" % (devpath, info))
         else:
-            log.warn("device '%s' not a block device. cannot resize: %s" %
-                     (devpath, info))
+            log.warning("device '%s' not a block device. cannot resize: %s" %
+                        (devpath, info))
         return None
     return devpath  # The writable block devpath
 
@@ -236,7 +210,7 @@ def handle(name, cfg, _cloud, log, args):
     resize_what = "/"
     result = util.get_mount_info(resize_what, log)
     if not result:
-        log.warn("Could not determine filesystem type of %s", resize_what)
+        log.warning("Could not determine filesystem type of %s", resize_what)
         return
 
     (devpth, fs_type, mount_point) = result
@@ -273,8 +247,8 @@ def handle(name, cfg, _cloud, log, args):
             break
 
     if not resizer:
-        log.warn("Not resizing unknown filesystem type %s for %s",
-                 fs_type, resize_what)
+        log.warning("Not resizing unknown filesystem type %s for %s",
+                    fs_type, resize_what)
         return
 
     resize_cmd = resizer(resize_what, devpth)
@@ -300,8 +274,8 @@ def handle(name, cfg, _cloud, log, args):
 
 def do_resize(resize_cmd, log):
     try:
-        util.subp(resize_cmd)
-    except util.ProcessExecutionError:
+        subp.subp(resize_cmd)
+    except subp.ProcessExecutionError:
         util.logexc(log, "Failed to resize filesystem (cmd=%s)", resize_cmd)
         raise
     # TODO(harlowja): Should we add a fsck check after this to make
