@@ -5,35 +5,39 @@
 # Author: Juerg Haefliger <juerg.haefliger@hp.com>
 #
 # This file is part of cloud-init. See LICENSE file for license information.
+"""Scripts User: Run user scripts"""
 
-"""
-Scripts User
-------------
-**Summary:** run user scripts
+import os
 
+from cloudinit import subp
+from cloudinit.config.schema import MetaSchema, get_meta_doc
+from cloudinit.distros import ALL_DISTROS
+from cloudinit.settings import PER_INSTANCE
+
+MODULE_DESCRIPTION = """\
 This module runs all user scripts. User scripts are not specified in the
 ``scripts`` directory in the datasource, but rather are present in the
 ``scripts`` dir in the instance configuration. Any cloud-config parts with a
 ``#!`` will be treated as a script and run. Scripts specified as cloud-config
 parts will be run in the order they are specified in the configuration.
 This module does not accept any config keys.
-
-**Internal name:** ``cc_scripts_user``
-
-**Module frequency:** per instance
-
-**Supported distros:** all
 """
 
-import os
+meta: MetaSchema = {
+    "id": "cc_scripts_user",
+    "name": "Scripts User",
+    "title": "Run user scripts",
+    "description": MODULE_DESCRIPTION,
+    "distros": [ALL_DISTROS],
+    "frequency": PER_INSTANCE,
+    "examples": [],
+    "activate_by_schema_keys": [],
+}
 
-from cloudinit import util
+__doc__ = get_meta_doc(meta)
 
-from cloudinit.settings import PER_INSTANCE
 
-frequency = PER_INSTANCE
-
-SCRIPT_SUBDIR = 'scripts'
+SCRIPT_SUBDIR = "scripts"
 
 
 def handle(name, _cfg, cloud, log, _args):
@@ -42,10 +46,15 @@ def handle(name, _cfg, cloud, log, _args):
     # go here...
     runparts_path = os.path.join(cloud.get_ipath_cur(), SCRIPT_SUBDIR)
     try:
-        util.runparts(runparts_path)
+        subp.runparts(runparts_path)
     except Exception:
-        log.warn("Failed to run module %s (%s in %s)",
-                 name, SCRIPT_SUBDIR, runparts_path)
+        log.warning(
+            "Failed to run module %s (%s in %s)",
+            name,
+            SCRIPT_SUBDIR,
+            runparts_path,
+        )
         raise
 
+
 # vi: ts=4 expandtab
