@@ -34,7 +34,7 @@ based on initial hostname.
 
 **Internal name:** ``cc_set_hostname``
 
-**Module frequency:** always
+**Module frequency:** per always
 
 **Supported distros:** all
 
@@ -48,8 +48,9 @@ based on initial hostname.
 
 import os
 
-from cloudinit import util
+
 from cloudinit.atomic_helper import write_json
+from cloudinit import util
 
 
 class SetHostnameError(Exception):
@@ -62,19 +63,16 @@ class SetHostnameError(Exception):
 
 def handle(name, cfg, cloud, log, _args):
     if util.get_cfg_option_bool(cfg, "preserve_hostname", False):
-        log.debug(
-            "Configuration option 'preserve_hostname' is set,"
-            " not setting the hostname in module %s",
-            name,
-        )
+        log.debug(("Configuration option 'preserve_hostname' is set,"
+                   " not setting the hostname in module %s"), name)
         return
 
     # Set prefer_fqdn_over_hostname value in distro
-    hostname_fqdn = util.get_cfg_option_bool(
-        cfg, "prefer_fqdn_over_hostname", None
-    )
+    hostname_fqdn = util.get_cfg_option_bool(cfg,
+                                             "prefer_fqdn_over_hostname",
+                                             None)
     if hostname_fqdn is not None:
-        cloud.distro.set_option("prefer_fqdn_over_hostname", hostname_fqdn)
+        cloud.distro.set_option('prefer_fqdn_over_hostname', hostname_fqdn)
 
     (hostname, fqdn) = util.get_hostname_fqdn(cfg, cloud)
     # Check for previous successful invocation of set-hostname
@@ -84,15 +82,14 @@ def handle(name, cfg, cloud, log, _args):
     # previous-hostname file which only contains the base hostname.
     # TODO consolidate previous-hostname and set-hostname artifact files and
     # distro._read_hostname implementation so we only validate  one artifact.
-    prev_fn = os.path.join(cloud.get_cpath("data"), "set-hostname")
+    prev_fn = os.path.join(cloud.get_cpath('data'), "set-hostname")
     prev_hostname = {}
     if os.path.exists(prev_fn):
         prev_hostname = util.load_json(util.load_file(prev_fn))
-    hostname_changed = hostname != prev_hostname.get(
-        "hostname"
-    ) or fqdn != prev_hostname.get("fqdn")
+    hostname_changed = (hostname != prev_hostname.get('hostname') or
+                        fqdn != prev_hostname.get('fqdn'))
     if not hostname_changed:
-        log.debug("No hostname changes. Skipping set-hostname")
+        log.debug('No hostname changes. Skipping set-hostname')
         return
     log.debug("Setting the hostname to %s (%s)", fqdn, hostname)
     try:
@@ -101,7 +98,6 @@ def handle(name, cfg, cloud, log, _args):
         msg = "Failed to set the hostname to %s (%s)" % (fqdn, hostname)
         util.logexc(log, msg)
         raise SetHostnameError("%s: %s" % (msg, e)) from e
-    write_json(prev_fn, {"hostname": hostname, "fqdn": fqdn})
-
+    write_json(prev_fn, {'hostname': hostname, 'fqdn': fqdn})
 
 # vi: ts=4 expandtab
