@@ -7,86 +7,25 @@
 # This file is part of cloud-init. See LICENSE file for license information.
 """Set Passwords: Set user passwords and enable/disable SSH password auth"""
 
+import logging
 import re
-from logging import Logger
 from string import ascii_letters, digits
-from textwrap import dedent
 from typing import List
 
-from cloudinit import features
-from cloudinit import log as logging
-from cloudinit import subp, util
+from cloudinit import features, subp, util
 from cloudinit.cloud import Cloud
-from cloudinit.config.schema import MetaSchema, get_meta_doc
+from cloudinit.config import Config
+from cloudinit.config.schema import MetaSchema
 from cloudinit.distros import ALL_DISTROS, Distro, ug_util
 from cloudinit.settings import PER_INSTANCE
 from cloudinit.ssh_util import update_ssh_config
 
-MODULE_DESCRIPTION = """\
-This module consumes three top-level config keys: ``ssh_pwauth``, ``chpasswd``
-and ``password``.
-
-The ``ssh_pwauth`` config key determines whether or not sshd will be configured
-to accept password authentication.
-
-The ``chpasswd`` config key accepts a dictionary containing either or both of
-``users`` and ``expire``. The ``users`` key is used to assign a password to a
-corresponding pre-existing user. The ``expire`` key is used to set
-whether to expire all user passwords specified by this module,
-such that a password will need to be reset on the user's next login.
-
-.. note::
-    Prior to cloud-init 22.3, the ``expire`` key only applies to plain text
-    (including ``RANDOM``) passwords. Post 22.3, the ``expire`` key applies to
-    both plain text and hashed passwords.
-
-``password`` config key is used to set the default user's password. It is
-ignored if the ``chpasswd`` ``users`` is used. Note: the ``list`` keyword is
-deprecated in favor of ``users``.
-"""
-
 meta: MetaSchema = {
     "id": "cc_set_passwords",
-    "name": "Set Passwords",
-    "title": "Set user passwords and enable/disable SSH password auth",
-    "description": MODULE_DESCRIPTION,
     "distros": [ALL_DISTROS],
     "frequency": PER_INSTANCE,
-    "examples": [
-        dedent(
-            """\
-            # Set a default password that would need to be changed
-            # at first login
-            ssh_pwauth: true
-            password: password1
-            """
-        ),
-        dedent(
-            """\
-            # Disable ssh password authentication
-            # Don't require users to change their passwords on next login
-            # Set the password for user1 to be 'password1' (OS does hashing)
-            # Set the password for user2 to a pre-hashed password
-            # Set the password for user3 to be a randomly generated password,
-            #   which will be written to the system console
-            ssh_pwauth: false
-            chpasswd:
-              expire: false
-              users:
-                - name: user1
-                  password: password1
-                  type: text
-                - name: user2
-                  password: $6$rounds=4096$5DJ8a9WMTEzIo5J4$Yms6imfeBvf3Yfu84mQBerh18l7OR1Wm1BJXZqFSpJ6BVas0AYJqIjP7czkOaAZHZi1kxQ5Y1IhgWN8K9NgxR1
-                - name: user3
-                  type: RANDOM
-            """  # noqa
-        ),
-    ],
     "activate_by_schema_keys": [],
-}
-
-__doc__ = get_meta_doc(meta)
+}  # type: ignore
 
 LOG = logging.getLogger(__name__)
 
@@ -107,6 +46,18 @@ def get_users_by_type(users_list: list, pw_type: str) -> list:
     )
 
 
+def _restart_ssh_daemon(distro, service):
+    try:
+        distro.manage_service("restart", service)
+        LOG.debug("Restarted the SSH daemon.")
+    except subp.ProcessExecutionError as e:
+        LOG.warning(
+            "'ssh_pwauth' configuration may not be applied. Cloud-init was "
+            "unable to restart SSH daemon due to error: '%s'",
+            e,
+        )
+
+
 def handle_ssh_pwauth(pw_auth, distro: Distro):
     """Apply sshd PasswordAuthentication changes.
 
@@ -116,55 +67,14 @@ def handle_ssh_pwauth(pw_auth, distro: Distro):
 
     @return: None"""
     service = distro.get_option("ssh_svcname", "ssh")
-    restart_ssh = True
-    try:
-        distro.manage_service("status", service)
-    except subp.ProcessExecutionError as e:
-        uses_systemd = distro.uses_systemd()
-        if not uses_systemd:
-            LOG.debug(
-                "Writing config 'ssh_pwauth: %s'. SSH service '%s'"
-                " will not be restarted because it is not running or not"
-                " available.",
-                pw_auth,
-                service,
-            )
-            restart_ssh = False
-        elif e.exit_code == 3:
-            # Service is not running. Write ssh config.
-            LOG.debug(
-                "Writing config 'ssh_pwauth: %s'. SSH service '%s'"
-                " will not be restarted because it is stopped.",
-                pw_auth,
-                service,
-            )
-            restart_ssh = False
-        elif e.exit_code == 4:
-            # Service status is unknown
-            LOG.warning(
-                "Ignoring config 'ssh_pwauth: %s'."
-                " SSH service '%s' is not installed.",
-                pw_auth,
-                service,
-            )
-            return
-        else:
-            LOG.warning(
-                "Ignoring config 'ssh_pwauth: %s'."
-                " SSH service '%s' is not available. Error: %s.",
-                pw_auth,
-                service,
-                e,
-            )
-            return
 
     cfg_name = "PasswordAuthentication"
 
     if isinstance(pw_auth, str):
-        LOG.warning(
-            "DEPRECATION: The 'ssh_pwauth' config key should be set to "
-            "a boolean value. The string format is deprecated and will be "
-            "removed in a future version of cloud-init."
+        util.deprecate(
+            deprecated="Using a string value for the 'ssh_pwauth' key",
+            deprecated_version="22.2",
+            extra_message="Use a boolean value with 'ssh_pwauth'.",
         )
     if util.is_true(pw_auth):
         cfg_val = "yes"
@@ -183,14 +93,24 @@ def handle_ssh_pwauth(pw_auth, distro: Distro):
         LOG.debug("No need to restart SSH service, %s not updated.", cfg_name)
         return
 
-    if restart_ssh:
-        distro.manage_service("restart", service)
-        LOG.debug("Restarted the SSH daemon.")
+    if distro.uses_systemd():
+        state = subp.subp(
+            [
+                "systemctl",
+                "show",
+                "--property",
+                "ActiveState",
+                "--value",
+                service,
+            ]
+        ).stdout.strip()
+        if state.lower() in ["active", "activating", "reloading"]:
+            _restart_ssh_daemon(distro, service)
     else:
-        LOG.debug("Not restarting SSH service: service is stopped.")
+        _restart_ssh_daemon(distro, service)
 
 
-def handle(_name, cfg: dict, cloud: Cloud, log: Logger, args: list):
+def handle(name: str, cfg: Config, cloud: Cloud, args: list) -> None:
     distro: Distro = cloud.distro
     if args:
         # if run from command line, and give args, wipe the chpasswd['list']
@@ -208,19 +128,21 @@ def handle(_name, cfg: dict, cloud: Cloud, log: Logger, args: list):
         chfg = cfg["chpasswd"]
         users_list = util.get_cfg_option_list(chfg, "users", default=[])
         if "list" in chfg and chfg["list"]:
-            log.warning(
-                "DEPRECATION: key 'lists' is now deprecated. Use 'users'."
+            util.deprecate(
+                deprecated="Config key 'lists'",
+                deprecated_version="22.3",
+                extra_message="Use 'users' instead.",
             )
             if isinstance(chfg["list"], list):
-                log.debug("Handling input for chpasswd as list.")
+                LOG.debug("Handling input for chpasswd as list.")
                 plist = util.get_cfg_option_list(chfg, "list", plist)
             else:
-                log.warning(
-                    "DEPRECATION: The chpasswd multiline string format is "
-                    "deprecated and will be removed from a future version of "
-                    "cloud-init. Use the list format instead."
+                util.deprecate(
+                    deprecated="The chpasswd multiline string",
+                    deprecated_version="22.2",
+                    extra_message="Use string type instead.",
                 )
-                log.debug("Handling input for chpasswd as multiline string.")
+                LOG.debug("Handling input for chpasswd as multiline string.")
                 multiline = util.get_cfg_option_str(chfg, "list")
                 if multiline:
                     plist = multiline.splitlines()
@@ -233,7 +155,7 @@ def handle(_name, cfg: dict, cloud: Cloud, log: Logger, args: list):
         if user:
             plist = ["%s:%s" % (user, password)]
         else:
-            log.warning("No default or defined user to change password for.")
+            LOG.warning("No default or defined user to change password for.")
 
     errors = []
     if plist or users_list:
@@ -273,22 +195,22 @@ def handle(_name, cfg: dict, cloud: Cloud, log: Logger, args: list):
                 users.append(u)
         if users:
             try:
-                log.debug("Changing password for %s:", users)
+                LOG.debug("Changing password for %s:", users)
                 distro.chpasswd(plist_in, hashed=False)
             except Exception as e:
                 errors.append(e)
                 util.logexc(
-                    log, "Failed to set passwords with chpasswd for %s", users
+                    LOG, "Failed to set passwords with chpasswd for %s", users
                 )
 
         if hashed_users:
             try:
-                log.debug("Setting hashed password for %s:", hashed_users)
+                LOG.debug("Setting hashed password for %s:", hashed_users)
                 distro.chpasswd(hashed_plist_in, hashed=True)
             except Exception as e:
                 errors.append(e)
                 util.logexc(
-                    log,
+                    LOG,
                     "Failed to set hashed passwords with chpasswd for %s",
                     hashed_users,
                 )
@@ -313,14 +235,14 @@ def handle(_name, cfg: dict, cloud: Cloud, log: Logger, args: list):
                     expired_users.append(u)
                 except Exception as e:
                     errors.append(e)
-                    util.logexc(log, "Failed to set 'expire' for %s", u)
+                    util.logexc(LOG, "Failed to set 'expire' for %s", u)
             if expired_users:
-                log.debug("Expired passwords for: %s users", expired_users)
+                LOG.debug("Expired passwords for: %s users", expired_users)
 
     handle_ssh_pwauth(cfg.get("ssh_pwauth"), distro)
 
     if len(errors):
-        log.debug("%s errors occurred, re-raising the last one", len(errors))
+        LOG.debug("%s errors occurred, re-raising the last one", len(errors))
         raise errors[-1]
 
 
