@@ -8,14 +8,11 @@
 #
 # This file is part of cloud-init. See LICENSE file for license information.
 
+import logging
 import os
 
-from cloudinit import handlers
-from cloudinit import log as logging
-from cloudinit import subp
-from cloudinit import util
-
-from cloudinit.settings import (PER_ALWAYS)
+from cloudinit import handlers, subp, util
+from cloudinit.settings import PER_ALWAYS
 
 LOG = logging.getLogger(__name__)
 
@@ -23,7 +20,7 @@ LOG = logging.getLogger(__name__)
 class BootHookPartHandler(handlers.Handler):
 
     # The content prefixes this handler understands.
-    prefixes = ['#cloud-boothook']
+    prefixes = ["#cloud-boothook"]
 
     def __init__(self, paths, datasource, **_kwargs):
         handlers.Handler.__init__(self, PER_ALWAYS)
@@ -35,8 +32,9 @@ class BootHookPartHandler(handlers.Handler):
     def _write_part(self, payload, filename):
         filename = util.clean_filename(filename)
         filepath = os.path.join(self.boothook_dir, filename)
-        contents = util.strip_prefix_suffix(util.dos2unix(payload),
-                                            prefix=self.prefixes[0])
+        contents = util.strip_prefix_suffix(
+            util.dos2unix(payload), prefix=self.prefixes[0]
+        )
         util.write_file(filepath, contents.lstrip(), 0o700)
         return filepath
 
@@ -48,12 +46,12 @@ class BootHookPartHandler(handlers.Handler):
         try:
             env = os.environ.copy()
             if self.instance_id is not None:
-                env['INSTANCE_ID'] = str(self.instance_id)
-            subp.subp([filepath], env=env)
+                env["INSTANCE_ID"] = str(self.instance_id)
+            LOG.debug("Executing boothook")
+            subp.subp([filepath], env=env, capture=False)
         except subp.ProcessExecutionError:
             util.logexc(LOG, "Boothooks script %s execution error", filepath)
         except Exception:
-            util.logexc(LOG, "Boothooks unknown error when running %s",
-                        filepath)
-
-# vi: ts=4 expandtab
+            util.logexc(
+                LOG, "Boothooks unknown error when running %s", filepath
+            )
