@@ -27,12 +27,14 @@ class InitramfsNetworkConfigSource(metaclass=abc.ABCMeta):
     """ABC for net config sources that read config written by initramfses"""
 
     @abc.abstractmethod
-    def is_applicable(self) -> bool:
+    def is_applicable(self):
+        # type: () -> bool
         """Is this initramfs config source applicable to the current system?"""
         pass
 
     @abc.abstractmethod
-    def render_config(self) -> dict:
+    def render_config(self):
+        # type: () -> dict
         """Render a v1 network config from the initramfs configuration"""
         pass
 
@@ -63,7 +65,8 @@ class KlibcNetworkConfigSource(InitramfsNetworkConfigSource):
                 if mac_addr:
                     self._mac_addrs[k] = mac_addr
 
-    def is_applicable(self) -> bool:
+    def is_applicable(self):
+        # type: () -> bool
         """
         Return whether this system has klibc initramfs network config or not
 
@@ -81,7 +84,8 @@ class KlibcNetworkConfigSource(InitramfsNetworkConfigSource):
                 return True
         return False
 
-    def render_config(self) -> dict:
+    def render_config(self):
+        # type: () -> dict
         return config_from_klibc_net_cfg(
             files=self._files, mac_addrs=self._mac_addrs,
         )
