@@ -14,22 +14,18 @@
 
 import collections
 import functools
+import logging
 import os
 import pwd
 import re
 import string
 
-from cloudinit import log as logging
-from cloudinit import net
-from cloudinit import sources
-from cloudinit import subp
-from cloudinit import util
-
+from cloudinit import atomic_helper, net, sources, subp, util
 
 LOG = logging.getLogger(__name__)
 
 DEFAULT_IID = "iid-dsopennebula"
-DEFAULT_PARSEUSER = 'nobody'
+DEFAULT_PARSEUSER = "nobody"
 CONTEXT_DISK_FILES = ["context.sh"]
 
 
@@ -40,7 +36,7 @@ class DataSourceOpenNebula(sources.DataSource):
     def __init__(self, sys_cfg, distro, paths):
         sources.DataSource.__init__(self, sys_cfg, distro, paths)
         self.seed = None
-        self.seed_dir = os.path.join(paths.seed_dir, 'opennebula')
+        self.seed_dir = os.path.join(paths.seed_dir, "opennebula")
 
     def __str__(self):
         root = sources.DataSource.__str__(self)
@@ -53,8 +49,8 @@ class DataSourceOpenNebula(sources.DataSource):
 
         # decide parseuser for context.sh shell reader
         parseuser = DEFAULT_PARSEUSER
-        if 'parseuser' in self.ds_cfg:
-            parseuser = self.ds_cfg.get('parseuser')
+        if "parseuser" in self.ds_cfg:
+            parseuser = self.ds_cfg.get("parseuser")
 
         candidates = [self.seed_dir]
         candidates.extend(find_candidate_devs())
@@ -90,29 +86,30 @@ class DataSourceOpenNebula(sources.DataSource):
             return False
 
         # merge fetched metadata with datasource defaults
-        md = results['metadata']
+        md = results["metadata"]
         md = util.mergemanydict([md, defaults])
 
         # check for valid user specified dsmode
         self.dsmode = self._determine_dsmode(
-            [results.get('DSMODE'), self.ds_cfg.get('dsmode')])
+            [results.get("DSMODE"), self.ds_cfg.get("dsmode")]
+        )
 
         if self.dsmode == sources.DSMODE_DISABLED:
             return False
 
         self.seed = seed
-        self.network = results.get('network-interfaces')
+        self.network = results.get("network-interfaces")
         self.metadata = md
-        self.userdata_raw = results.get('userdata')
+        self.userdata_raw = results.get("userdata")
         return True
 
     def _get_subplatform(self):
         """Return the subplatform metadata source details."""
         if self.seed_dir in self.seed:
-            subplatform_type = 'seed-dir'
+            subplatform_type = "seed-dir"
         else:
-            subplatform_type = 'config-disk'
-        return '%s (%s)' % (subplatform_type, self.seed)
+            subplatform_type = "config-disk"
+        return "%s (%s)" % (subplatform_type, self.seed)
 
     @property
     def network_config(self):
@@ -138,38 +135,41 @@ class BrokenContextDiskDir(Exception):
     pass
 
 
-class OpenNebulaNetwork(object):
+class OpenNebulaNetwork:
     def __init__(self, context, distro, system_nics_by_mac=None):
         self.context = context
         if system_nics_by_mac is None:
             system_nics_by_mac = get_physical_nics_by_mac(distro)
         self.ifaces = collections.OrderedDict(
-            [k for k in sorted(system_nics_by_mac.items(),
-                               key=lambda k: net.natural_sort_key(k[1]))])
+            [
+                k
+                for k in sorted(
+                    system_nics_by_mac.items(),
+                    key=lambda k: net.natural_sort_key(k[1]),
+                )
+            ]
+        )
 
         # OpenNebula 4.14+ provide macaddr for ETHX in variable ETH_MAC.
         # context_devname provides {mac.lower():ETHX, mac2.lower():ETHX}
         self.context_devname = {}
         for k, v in context.items():
-            m = re.match(r'^(.+)_MAC$', k)
+            m = re.match(r"^(.+)_MAC$", k)
             if m:
                 self.context_devname[v.lower()] = m.group(1)
 
     def mac2ip(self, mac):
-        return '.'.join([str(int(c, 16)) for c in mac.split(':')[2:]])
-
-    def mac2network(self, mac):
-        return self.mac2ip(mac).rpartition(".")[0] + ".0"
+        return ".".join([str(int(c, 16)) for c in mac.split(":")[2:]])
 
     def get_nameservers(self, dev):
         nameservers = {}
         dns = self.get_field(dev, "dns", "").split()
-        dns.extend(self.context.get('DNS', "").split())
+        dns.extend(self.context.get("DNS", "").split())
         if dns:
-            nameservers['addresses'] = dns
+            nameservers["addresses"] = dns
         search_domain = self.get_field(dev, "search_domain", "").split()
         if search_domain:
-            nameservers['search'] = search_domain
+            nameservers["search"] = search_domain
         return nameservers
 
     def get_mtu(self, dev):
@@ -195,27 +195,36 @@ class OpenNebulaNetwork(object):
         return self.get_field(dev, "gateway")
 
     def get_gateway6(self, dev):
-        return self.get_field(dev, "gateway6")
+        # OpenNebula 6.1.80 introduced new context parameter ETHx_IP6_GATEWAY
+        # to replace old ETHx_GATEWAY6. Old ETHx_GATEWAY6 will be removed in
+        # OpenNebula 6.4.0 (https://github.com/OpenNebula/one/issues/5536).
+        return self.get_field(
+            dev, "ip6_gateway", self.get_field(dev, "gateway6")
+        )
 
     def get_mask(self, dev):
         return self.get_field(dev, "mask", "255.255.255.0")
 
-    def get_network(self, dev, mac):
-        return self.get_field(dev, "network", self.mac2network(mac))
-
     def get_field(self, dev, name, default=None):
         """return the field name in context for device dev.
 
         context stores <dev>_<NAME> (example: eth0_DOMAIN).
         an empty string for value will return default."""
-        val = self.context.get('_'.join((dev, name,)).upper())
+        val = self.context.get(
+            "_".join(
+                (
+                    dev,
+                    name,
+                )
+            ).upper()
+        )
         # allow empty string to return the default.
         return default if val in (None, "") else val
 
     def gen_conf(self):
         netconf = {}
-        netconf['version'] = 2
-        netconf['ethernets'] = {}
+        netconf["version"] = 2
+        netconf["ethernets"] = {}
 
         ethernets = {}
         for mac, dev in self.ifaces.items():
@@ -228,46 +237,46 @@ class OpenNebulaNetwork(object):
             devconf = {}
 
             # Set MAC address
-            devconf['match'] = {'macaddress': mac}
+            devconf["match"] = {"macaddress": mac}
 
             # Set IPv4 address
-            devconf['addresses'] = []
+            devconf["addresses"] = []
             mask = self.get_mask(c_dev)
-            prefix = str(net.mask_to_net_prefix(mask))
-            devconf['addresses'].append(
-                self.get_ip(c_dev, mac) + '/' + prefix)
+            prefix = str(net.ipv4_mask_to_net_prefix(mask))
+            devconf["addresses"].append(self.get_ip(c_dev, mac) + "/" + prefix)
 
             # Set IPv6 Global and ULA address
             addresses6 = self.get_ip6(c_dev)
             if addresses6:
                 prefix6 = self.get_ip6_prefix(c_dev)
-                devconf['addresses'].extend(
-                    [i + '/' + prefix6 for i in addresses6])
+                devconf["addresses"].extend(
+                    [i + "/" + prefix6 for i in addresses6]
+                )
 
             # Set IPv4 default gateway
             gateway = self.get_gateway(c_dev)
             if gateway:
-                devconf['gateway4'] = gateway
+                devconf["gateway4"] = gateway
 
             # Set IPv6 default gateway
             gateway6 = self.get_gateway6(c_dev)
             if gateway6:
-                devconf['gateway6'] = gateway6
+                devconf["gateway6"] = gateway6
 
             # Set DNS servers and search domains
             nameservers = self.get_nameservers(c_dev)
             if nameservers:
-                devconf['nameservers'] = nameservers
+                devconf["nameservers"] = nameservers
 
             # Set MTU size
             mtu = self.get_mtu(c_dev)
             if mtu:
-                devconf['mtu'] = mtu
+                devconf["mtu"] = mtu
 
             ethernets[dev] = devconf
 
-        netconf['ethernets'] = ethernets
-        return(netconf)
+        netconf["ethernets"] = ethernets
+        return netconf
 
 
 def find_candidate_devs():
@@ -275,7 +284,7 @@ def find_candidate_devs():
     Return a list of devices that may contain the context disk.
     """
     combined = []
-    for f in ('LABEL=CONTEXT', 'LABEL=CDROM', 'TYPE=iso9660'):
+    for f in ("LABEL=CONTEXT", "LABEL=CDROM", "TYPE=iso9660"):
         devs = util.find_devs_with(f)
         devs.sort()
         for d in devs:
@@ -286,16 +295,17 @@ def find_candidate_devs():
 
 
 def switch_user_cmd(user):
-    return ['sudo', '-u', user]
+    return ["sudo", "-u", user]
 
 
-def parse_shell_config(content, keylist=None, bash=None, asuser=None,
-                       switch_user_cb=None):
+def parse_shell_config(
+    content, keylist=None, bash=None, asuser=None, switch_user_cb=None
+):
 
     if isinstance(bash, str):
         bash = [bash]
     elif bash is None:
-        bash = ['bash', '-e']
+        bash = ["bash", "-e"]
 
     if switch_user_cb is None:
         switch_user_cb = switch_user_cmd
@@ -309,17 +319,24 @@ def parse_shell_config(content, keylist=None, bash=None, asuser=None,
         keylist = allvars
         keylist_in = []
 
-    setup = '\n'.join(('__v="";', '',))
+    setup = "\n".join(
+        (
+            '__v="";',
+            "",
+        )
+    )
 
     def varprinter(vlist):
         # output '\0'.join(['_start_', key=value NULL for vars in vlist]
-        return '\n'.join((
-            'printf "%s\\0" _start_',
-            'for __v in %s; do' % ' '.join(vlist),
-            '   printf "%s=%s\\0" "$__v" "${!__v}";',
-            'done',
-            ''
-        ))
+        return "\n".join(
+            (
+                'printf "%s\\0" _start_',
+                "for __v in %s; do" % " ".join(vlist),
+                '   printf "%s=%s\\0" "$__v" "${!__v}";',
+                "done",
+                "",
+            )
+        )
 
     # the rendered 'bcmd' is bash syntax that does
     # setup: declare variables we use (so they show up in 'all')
@@ -332,12 +349,15 @@ def parse_shell_config(content, keylist=None, bash=None, asuser=None,
     #   key=value (for each preset variable)
     #   literal '_start_'
     #   key=value (for each post set variable)
-    bcmd = ('unset IFS\n' +
-            setup +
-            varprinter(allvars) +
-            '{\n%s\n\n:\n} > /dev/null\n' % content +
-            'unset IFS\n' +
-            varprinter(keylist) + "\n")
+    bcmd = (
+        "unset IFS\n"
+        + setup
+        + varprinter(allvars)
+        + "{\n%s\n\n:\n} > /dev/null\n" % content
+        + "unset IFS\n"
+        + varprinter(keylist)
+        + "\n"
+    )
 
     cmd = []
     if asuser is not None:
@@ -349,8 +369,15 @@ def parse_shell_config(content, keylist=None, bash=None, asuser=None,
 
     # exclude vars in bash that change on their own or that we used
     excluded = (
-        "EPOCHREALTIME", "EPOCHSECONDS", "RANDOM", "LINENO", "SECONDS", "_",
-        "__v")
+        "EPOCHREALTIME",
+        "EPOCHSECONDS",
+        "RANDOM",
+        "LINENO",
+        "SECONDS",
+        "_",
+        "SRANDOM",
+        "__v",
+    )
     preset = {}
     ret = {}
     target = None
@@ -363,8 +390,9 @@ def parse_shell_config(content, keylist=None, bash=None, asuser=None,
             (key, val) = line.split("=", 1)
             if target is preset:
                 preset[key] = val
-            elif (key not in excluded and
-                  (key in keylist_in or preset.get(key) != val)):
+            elif key not in excluded and (
+                key in keylist_in or preset.get(key) != val
+            ):
                 ret[key] = val
         except ValueError:
             if line != "_start_":
@@ -393,7 +421,7 @@ def read_context_disk_dir(source_dir, distro, asuser=None):
         raise NonContextDiskDir("%s: %s" % (source_dir, "no files found"))
 
     context = {}
-    results = {'userdata': None, 'metadata': {}}
+    results = {"userdata": None, "metadata": {}}
 
     if "context.sh" in found:
         if asuser is not None:
@@ -402,10 +430,11 @@ def read_context_disk_dir(source_dir, distro, asuser=None):
             except KeyError as e:
                 raise BrokenContextDiskDir(
                     "configured user '{user}' does not exist".format(
-                        user=asuser)
+                        user=asuser
+                    )
                 ) from e
         try:
-            path = os.path.join(source_dir, 'context.sh')
+            path = os.path.join(source_dir, "context.sh")
             content = util.load_file(path)
             context = parse_shell_config(content, asuser=asuser)
         except subp.ProcessExecutionError as e:
@@ -422,7 +451,7 @@ def read_context_disk_dir(source_dir, distro, asuser=None):
     if not context:
         return results
 
-    results['metadata'] = context
+    results["metadata"] = context
 
     # process single or multiple SSH keys
     ssh_key_var = None
@@ -433,40 +462,41 @@ def read_context_disk_dir(source_dir, distro, asuser=None):
 
     if ssh_key_var:
         lines = context.get(ssh_key_var).splitlines()
-        results['metadata']['public-keys'] = [
+        results["metadata"]["public-keys"] = [
             line for line in lines if len(line) and not line.startswith("#")
         ]
 
     # custom hostname -- try hostname or leave cloud-init
     # itself create hostname from IP address later
-    for k in ('HOSTNAME', 'PUBLIC_IP', 'IP_PUBLIC', 'ETH0_IP'):
+    for k in ("SET_HOSTNAME", "HOSTNAME", "PUBLIC_IP", "IP_PUBLIC", "ETH0_IP"):
         if k in context:
-            results['metadata']['local-hostname'] = context[k]
+            results["metadata"]["local-hostname"] = context[k]
             break
 
     # raw user data
     if "USER_DATA" in context:
-        results['userdata'] = context["USER_DATA"]
+        results["userdata"] = context["USER_DATA"]
     elif "USERDATA" in context:
-        results['userdata'] = context["USERDATA"]
+        results["userdata"] = context["USERDATA"]
 
     # b64decode user data if necessary (default)
-    if 'userdata' in results:
-        encoding = context.get('USERDATA_ENCODING',
-                               context.get('USER_DATA_ENCODING'))
+    if "userdata" in results:
+        encoding = context.get(
+            "USERDATA_ENCODING", context.get("USER_DATA_ENCODING")
+        )
         if encoding == "base64":
             try:
-                results['userdata'] = util.b64d(results['userdata'])
+                results["userdata"] = atomic_helper.b64d(results["userdata"])
             except TypeError:
                 LOG.warning("Failed base64 decoding of userdata")
 
     # generate Network Configuration v2
     # only if there are any required context variables
     # http://docs.opennebula.org/5.4/operation/references/template.html#context-section
-    ipaddr_keys = [k for k in context if re.match(r'^ETH\d+_IP.*$', k)]
+    ipaddr_keys = [k for k in context if re.match(r"^ETH\d+_IP.*$", k)]
     if ipaddr_keys:
         onet = OpenNebulaNetwork(context, distro)
-        results['network-interfaces'] = onet.gen_conf()
+        results["network-interfaces"] = onet.gen_conf()
 
     return results
 
@@ -483,12 +513,10 @@ DataSourceOpenNebulaNet = DataSourceOpenNebula
 
 # Used to match classes to dependencies
 datasources = [
-    (DataSourceOpenNebula, (sources.DEP_FILESYSTEM, )),
+    (DataSourceOpenNebula, (sources.DEP_FILESYSTEM,)),
 ]
 
 
 # Return a list of data sources that match this set of dependencies
 def get_datasource_list(depends):
     return sources.list_from_depends(depends, datasources)
-
-# vi: ts=4 expandtab
