@@ -4,20 +4,20 @@
 #
 # This file is part of cloud-init. See LICENSE file for license information.
 
+import logging
 import time
 
-from cloudinit import dmi
-from cloudinit import log as logging
-from cloudinit import sources, url_helper, util
+from cloudinit import dmi, sources, url_helper, util
 from cloudinit.event import EventScope, EventType
-from cloudinit.net.dhcp import EphemeralDHCPv4, NoDHCPLeaseError
+from cloudinit.net.dhcp import NoDHCPLeaseError
+from cloudinit.net.ephemeral import EphemeralDHCPv4
 from cloudinit.sources import DataSourceOracle as oracle
 from cloudinit.sources.helpers import openstack
 
 LOG = logging.getLogger(__name__)
 
 # Various defaults/constants...
-DEF_MD_URL = "http://169.254.169.254"
+DEF_MD_URLS = ["http://[fe80::a9fe:a9fe]", "http://169.254.169.254"]
 DEFAULT_IID = "iid-dsopenstack"
 DEFAULT_METADATA = {
     "instance-id": DEFAULT_IID,
@@ -31,8 +31,13 @@ DMI_ASSET_TAG_OPENTELEKOM = "OpenTelekomCloud"
 # See github.com/sapcc/helm-charts/blob/master/openstack/nova/values.yaml
 # -> compute.defaults.vmware.smbios_asset_tag for this value
 DMI_ASSET_TAG_SAPCCLOUD = "SAP CCloud VM"
+DMI_ASSET_TAG_HUAWEICLOUD = "HUAWEICLOUD"
 VALID_DMI_ASSET_TAGS = VALID_DMI_PRODUCT_NAMES
-VALID_DMI_ASSET_TAGS += [DMI_ASSET_TAG_OPENTELEKOM, DMI_ASSET_TAG_SAPCCLOUD]
+VALID_DMI_ASSET_TAGS += [
+    DMI_ASSET_TAG_HUAWEICLOUD,
+    DMI_ASSET_TAG_OPENTELEKOM,
+    DMI_ASSET_TAG_SAPCCLOUD,
+]
 
 
 class DataSourceOpenStack(openstack.SourceMixin, sources.DataSource):
@@ -68,7 +73,7 @@ class DataSourceOpenStack(openstack.SourceMixin, sources.DataSource):
         return mstr
 
     def wait_for_metadata_service(self):
-        urls = self.ds_cfg.get("metadata_urls", [DEF_MD_URL])
+        urls = self.ds_cfg.get("metadata_urls", DEF_MD_URLS)
         filtered = [x for x in urls if util.is_resolvable_url(x)]
         if set(filtered) != set(urls):
             LOG.debug(
@@ -79,7 +84,7 @@ class DataSourceOpenStack(openstack.SourceMixin, sources.DataSource):
             urls = filtered
         else:
             LOG.warning("Empty metadata url list! using default list")
-            urls = [DEF_MD_URL]
+            urls = DEF_MD_URLS
 
         md_urls = []
         url2base = {}
@@ -94,6 +99,7 @@ class DataSourceOpenStack(openstack.SourceMixin, sources.DataSource):
             urls=md_urls,
             max_wait=url_params.max_wait_seconds,
             timeout=url_params.timeout_seconds,
+            connect_synchronously=False,
         )
         if avail_url:
             LOG.debug("Using metadata source: '%s'", url2base[avail_url])
@@ -117,11 +123,10 @@ class DataSourceOpenStack(openstack.SourceMixin, sources.DataSource):
         if self._network_config != sources.UNSET:
             return self._network_config
 
-        # Xenial, Artful and Bionic will not provide
+        # RELEASE_BLOCKER: SRU to Xenial and Artful SRU should not provide
         # network_config by default unless configured in /etc/cloud/cloud.cfg*.
         # Patch Xenial and Artful before release to default to False.
-        if util.is_false(self.ds_cfg.get("apply_network_config", False)):
-
+        if util.is_false(self.ds_cfg.get("apply_network_config", True)):
             self._network_config = None
             return self._network_config
         if self.network_json == sources.UNSET:
@@ -144,13 +149,11 @@ class DataSourceOpenStack(openstack.SourceMixin, sources.DataSource):
             False when unable to contact metadata service or when metadata
             format is invalid or disabled.
         """
-        oracle_considered = "Oracle" in self.sys_cfg.get("datasource_list")
-        if not detect_openstack(accept_oracle=not oracle_considered):
-            return False
 
         if self.perform_dhcp_setup:  # Setup networking in init-local stage.
             try:
-                with EphemeralDHCPv4(self.fallback_interface):
+
+                with EphemeralDHCPv4(self.distro, self.fallback_interface):
                     results = util.log_time(
                         logfunc=LOG.debug,
                         msg="Crawl of metadata service",
@@ -238,6 +241,24 @@ class DataSourceOpenStack(openstack.SourceMixin, sources.DataSource):
             raise sources.InvalidMetaDataException(msg) from e
         return result
 
+    def ds_detect(self):
+        """Return True when a potential OpenStack platform is detected."""
+        accept_oracle = "Oracle" in self.sys_cfg.get("datasource_list")
+        if not util.is_x86():
+            # Non-Intel cpus don't properly report dmi product names
+            return True
+
+        product_name = dmi.read_dmi_data("system-product-name")
+        if product_name in VALID_DMI_PRODUCT_NAMES:
+            return True
+        elif dmi.read_dmi_data("chassis-asset-tag") in VALID_DMI_ASSET_TAGS:
+            return True
+        elif accept_oracle and oracle._is_platform_viable():
+            return True
+        elif util.get_proc_env(1).get("product_name") == DMI_PRODUCT_NOVA:
+            return True
+        return False
+
 
 class DataSourceOpenStackLocal(DataSourceOpenStack):
     """Run in init-local using a dhcp discovery prior to metadata crawl.
@@ -258,22 +279,6 @@ def read_metadata_service(base_url, ssl_details=None, timeout=5, retries=5):
     return reader.read_v2()
 
 
-def detect_openstack(accept_oracle=False):
-    """Return True when a potential OpenStack platform is detected."""
-    if not util.is_x86():
-        return True  # Non-Intel cpus don't properly report dmi product names
-    product_name = dmi.read_dmi_data("system-product-name")
-    if product_name in VALID_DMI_PRODUCT_NAMES:
-        return True
-    elif dmi.read_dmi_data("chassis-asset-tag") in VALID_DMI_ASSET_TAGS:
-        return True
-    elif accept_oracle and oracle._is_platform_viable():
-        return True
-    elif util.get_proc_env(1).get("product_name") == DMI_PRODUCT_NOVA:
-        return True
-    return False
-
-
 # Used to match classes to dependencies
 datasources = [
     (DataSourceOpenStackLocal, (sources.DEP_FILESYSTEM,)),
@@ -284,6 +289,3 @@ datasources = [
 # Return a list of data sources that match this set of dependencies
 def get_datasource_list(depends):
     return sources.list_from_depends(depends, datasources)
-
-
-# vi: ts=4 expandtab
