@@ -30,9 +30,11 @@ import random
 import re
 import socket
 
-from cloudinit import dmi
+import serial
+
+from cloudinit import atomic_helper, dmi
 from cloudinit import log as logging
-from cloudinit import serial, sources, subp, util
+from cloudinit import sources, subp, util
 from cloudinit.event import EventScope, EventType
 
 LOG = logging.getLogger(__name__)
@@ -368,7 +370,7 @@ class JoyentMetadataTimeoutException(JoyentMetadataFetchException):
     pass
 
 
-class JoyentMetadataClient(object):
+class JoyentMetadataClient:
     """
     A client implementing v2 of the Joyent Metadata Protocol Specification.
 
@@ -417,7 +419,7 @@ class JoyentMetadataClient(object):
         if not frame_data.get("payload", None):
             LOG.debug("No value found.")
             return None
-        value = util.b64d(frame_data["payload"])
+        value = atomic_helper.b64d(frame_data["payload"])
         LOG.debug('Value "%s" found.', value)
         return value
 
@@ -524,9 +526,6 @@ class JoyentMetadataClient(object):
         ).decode()
         return self.request(rtype="PUT", param=param)
 
-    def delete(self, key):
-        return self.request(rtype="DELETE", param=key)
-
     def close_transport(self):
         if self.fp:
             self.fp.close()
@@ -711,8 +710,7 @@ class JoyentMetadataLegacySerialClient(JoyentMetadataSerialClient):
         if self.is_b64_encoded(key):
             try:
                 val = base64.b64decode(val.encode()).decode()
-            # Bogus input produces different errors in Python 2 and 3
-            except (TypeError, binascii.Error):
+            except binascii.Error:
                 LOG.warning("Failed base64 decoding key '%s': %s", key, val)
 
         if strip:
@@ -1049,7 +1047,7 @@ if __name__ == "__main__":
 
         return data[key]
 
-    data = {}
+    data: dict = {}
     for key in keys:
         load_key(client=jmc, key=key, data=data)
 
