@@ -5,7 +5,7 @@ oauthlib.parameters
 
 This module contains methods related to `section 3.5`_ of the OAuth 1.0a spec.
 
-.. _`section 3.5`: http://tools.ietf.org/html/rfc5849#section-3.5
+.. _`section 3.5`: https://tools.ietf.org/html/rfc5849#section-3.5
 """
 from __future__ import absolute_import, unicode_literals
 
@@ -15,7 +15,7 @@ from . import utils
 
 try:
     from urlparse import urlparse, urlunparse
-except ImportError:
+except ImportError:  # noqa
     from urllib.parse import urlparse, urlunparse
 
 
@@ -42,8 +42,8 @@ def prepare_headers(oauth_params, headers=None, realm=None):
             oauth_version="1.0"
 
 
-    .. _`section 3.5.1`: http://tools.ietf.org/html/rfc5849#section-3.5.1
-    .. _`RFC2617`: http://tools.ietf.org/html/rfc2617
+    .. _`section 3.5.1`: https://tools.ietf.org/html/rfc5849#section-3.5.1
+    .. _`RFC2617`: https://tools.ietf.org/html/rfc2617
     """
     headers = headers or {}
 
@@ -54,7 +54,7 @@ def prepare_headers(oauth_params, headers=None, realm=None):
         # 1.  Parameter names and values are encoded per Parameter Encoding
         #     (`Section 3.6`_)
         #
-        # .. _`Section 3.6`: http://tools.ietf.org/html/rfc5849#section-3.6
+        # .. _`Section 3.6`: https://tools.ietf.org/html/rfc5849#section-3.6
         escaped_name = utils.escape(oauth_parameter_name)
         escaped_value = utils.escape(value)
 
@@ -68,14 +68,14 @@ def prepare_headers(oauth_params, headers=None, realm=None):
     # 3.  Parameters are separated by a "," character (ASCII code 44) and
     #     OPTIONAL linear whitespace per `RFC2617`_.
     #
-    # .. _`RFC2617`: http://tools.ietf.org/html/rfc2617
+    # .. _`RFC2617`: https://tools.ietf.org/html/rfc2617
     authorization_header_parameters = ', '.join(
         authorization_header_parameters_parts)
 
     # 4.  The OPTIONAL "realm" parameter MAY be added and interpreted per
     #     `RFC2617 section 1.2`_.
     #
-    # .. _`RFC2617 section 1.2`: http://tools.ietf.org/html/rfc2617#section-1.2
+    # .. _`RFC2617 section 1.2`: https://tools.ietf.org/html/rfc2617#section-1.2
     if realm:
         # NOTE: realm should *not* be escaped
         authorization_header_parameters = ('realm="%s", ' % realm +
@@ -98,8 +98,8 @@ def _append_params(oauth_params, params):
 
     Per `section 3.5.2`_ and `3.5.3`_ of the spec.
 
-    .. _`section 3.5.2`: http://tools.ietf.org/html/rfc5849#section-3.5.2
-    .. _`3.5.3`: http://tools.ietf.org/html/rfc5849#section-3.5.3
+    .. _`section 3.5.2`: https://tools.ietf.org/html/rfc5849#section-3.5.2
+    .. _`3.5.3`: https://tools.ietf.org/html/rfc5849#section-3.5.3
 
     """
     merged = list(params)
@@ -117,7 +117,7 @@ def prepare_form_encoded_body(oauth_params, body):
 
     Per `section 3.5.2`_ of the spec.
 
-    .. _`section 3.5.2`: http://tools.ietf.org/html/rfc5849#section-3.5.2
+    .. _`section 3.5.2`: https://tools.ietf.org/html/rfc5849#section-3.5.2
 
     """
     # append OAuth params to the existing body
@@ -129,7 +129,7 @@ def prepare_request_uri_query(oauth_params, uri):
 
     Per `section 3.5.3`_ of the spec.
 
-    .. _`section 3.5.3`: http://tools.ietf.org/html/rfc5849#section-3.5.3
+    .. _`section 3.5.3`: https://tools.ietf.org/html/rfc5849#section-3.5.3
 
     """
     # append OAuth params to the existing set of query components
