@@ -16,10 +16,12 @@ import logging
 import codecs
 import errno
 import stat
+import re
 from datetime import datetime
 from threading import Lock
 
-from sos.utilities import sos_get_command_output, is_executable
+from importlib.util import find_spec
+from sos.utilities import sos_get_command_output
 
 try:
     import selinux
@@ -71,7 +73,7 @@ class Archive(object):
     # this is our contract to clients of the Archive class hierarchy.
     # All sub-classes need to implement these methods (or inherit concrete
     # implementations from a parent class.
-    def add_file(self, src, dest=None):
+    def add_file(self, src, dest=None, force=False):
         raise NotImplementedError
 
     def add_string(self, content, dest, mode='w'):
@@ -134,6 +136,9 @@ class FileCacheArchive(Archive):
     def __init__(self, name, tmpdir, policy, threads, enc_opts, sysroot,
                  manifest=None):
         self._name = name
+        # truncate the name just relative to the tmpdir in case of full path
+        if os.path.commonprefix([self._name, tmpdir]) == tmpdir:
+            self._name = os.path.relpath(name, tmpdir)
         self._tmp_dir = tmpdir
         self._policy = policy
         self._threads = threads
@@ -152,7 +157,7 @@ class FileCacheArchive(Archive):
         return (os.path.join(self._archive_root, name))
 
     def join_sysroot(self, path):
-        if path.startswith(self.sysroot):
+        if not self.sysroot or path.startswith(self.sysroot):
             return path
         if path[0] == os.sep:
             path = path[1:]
@@ -250,7 +255,7 @@ class FileCacheArchive(Archive):
 
         return dest
 
-    def _check_path(self, src, path_type, dest=None, force=False):
+    def check_path(self, src, path_type, dest=None, force=False):
         """Check a new destination path in the archive.
 
             Since it is possible for multiple plugins to collect the same
@@ -325,12 +330,26 @@ class FileCacheArchive(Archive):
             return None
         return dest
 
-    def add_file(self, src, dest=None):
+    def _copy_attributes(self, src, dest):
+        # copy file attributes, skip SELinux xattrs for /sys and /proc
+        try:
+            stat = os.stat(src)
+            if src.startswith("/sys/") or src.startswith("/proc/"):
+                shutil.copymode(src, dest)
+                os.utime(dest, ns=(stat.st_atime_ns, stat.st_mtime_ns))
+            else:
+                shutil.copystat(src, dest)
+            os.chown(dest, stat.st_uid, stat.st_gid)
+        except Exception as e:
+            self.log_debug("caught '%s' setting attributes of '%s'"
+                           % (e, dest))
+
+    def add_file(self, src, dest=None, force=False):
         with self._path_lock:
             if not dest:
                 dest = src
 
-            dest = self._check_path(dest, P_FILE)
+            dest = self.check_path(dest, P_FILE, force=force)
             if not dest:
                 return
 
@@ -347,18 +366,7 @@ class FileCacheArchive(Archive):
                     else:
                         self.log_info("File %s not collected: '%s'" % (src, e))
 
-                # copy file attributes, skip SELinux xattrs for /sys and /proc
-                try:
-                    stat = os.stat(src)
-                    if src.startswith("/sys/") or src.startswith("/proc/"):
-                        shutil.copymode(src, dest)
-                        os.utime(dest, ns=(stat.st_atime_ns, stat.st_mtime_ns))
-                    else:
-                        shutil.copystat(src, dest)
-                    os.chown(dest, stat.st_uid, stat.st_gid)
-                except Exception as e:
-                    self.log_debug("caught '%s' setting attributes of '%s'"
-                                   % (e, dest))
+                self._copy_attributes(src, dest)
                 file_name = "'%s'" % src
             else:
                 # Open file case: first rewind the file to obtain
@@ -380,24 +388,20 @@ class FileCacheArchive(Archive):
             # over any exixting content in the archive, since it is used by
             # the Plugin postprocessing hooks to perform regex substitution
             # on file content.
-            dest = self._check_path(dest, P_FILE, force=True)
+            dest = self.check_path(dest, P_FILE, force=True)
 
             f = codecs.open(dest, mode, encoding='utf-8')
             if isinstance(content, bytes):
                 content = content.decode('utf8', 'ignore')
             f.write(content)
             if os.path.exists(src):
-                try:
-                    shutil.copystat(src, dest)
-                except OSError as e:
-                    self.log_error("Unable to add '%s' to archive: %s" %
-                                   (dest, e))
+                self._copy_attributes(src, dest)
             self.log_debug("added string at '%s' to FileCacheArchive '%s'"
                            % (src, self._archive_root))
 
     def add_binary(self, content, dest):
         with self._path_lock:
-            dest = self._check_path(dest, P_FILE)
+            dest = self.check_path(dest, P_FILE)
             if not dest:
                 return
 
@@ -409,7 +413,7 @@ class FileCacheArchive(Archive):
     def add_link(self, source, link_name):
         self.log_debug("adding symlink at '%s' -> '%s'" % (link_name, source))
         with self._path_lock:
-            dest = self._check_path(link_name, P_LINK)
+            dest = self.check_path(link_name, P_LINK)
             if not dest:
                 return
 
@@ -476,7 +480,6 @@ class FileCacheArchive(Archive):
             else:
                 self.log_debug("No link follow up: source=%s link_name=%s" %
                                (source, link_name))
-        self.log_debug("leaving add_link()")
 
     def add_dir(self, path):
         """Create a directory in the archive.
@@ -485,10 +488,10 @@ class FileCacheArchive(Archive):
         """
         # Establish path structure
         with self._path_lock:
-            self._check_path(path, P_DIR)
+            self.check_path(path, P_DIR)
 
     def add_node(self, path, mode, device):
-        dest = self._check_path(path, P_NODE)
+        dest = self.check_path(path, P_NODE)
         if not dest:
             return
 
@@ -501,7 +504,7 @@ class FileCacheArchive(Archive):
                     self.log_info("add_node: %s - mknod '%s'" % (msg, dest))
                     return
                 raise e
-            shutil.copystat(path, dest)
+            self._copy_attributes(path, dest)
 
     def name_max(self):
         if 'PC_NAME_MAX' in os.pathconf_names:
@@ -559,17 +562,16 @@ class FileCacheArchive(Archive):
     def finalize(self, method):
         self.log_info("finalizing archive '%s' using method '%s'"
                       % (self._archive_root, method))
-        self._build_archive()
+        try:
+            res = self._build_archive(method)
+        except Exception as err:
+            self.log_error("An error occurred compressing the archive: %s"
+                           % err)
+            return self.name()
+
         self.cleanup()
         self.log_info("built archive at '%s' (size=%d)" % (self._archive_name,
                       os.stat(self._archive_name).st_size))
-        self.method = method
-        try:
-            res = self._compress()
-        except Exception as e:
-            exp_msg = "An error occurred compressing the archive: "
-            self.log_error("%s %s" % (exp_msg, e))
-            return self.name()
 
         if self.enc_opts['encrypt']:
             try:
@@ -654,16 +656,19 @@ class TarFileArchive(FileCacheArchive):
     # this can be used to set permissions if using the
     # tarfile.add() interface to add directory trees.
     def copy_permissions_filter(self, tarinfo):
-        orig_path = tarinfo.name[len(os.path.split(self._name)[-1]):]
+        orig_path = tarinfo.name[len(os.path.split(self._archive_root)[-1]):]
         if not orig_path:
             orig_path = self._archive_root
+        skips = ['/version.txt$', '/sos_logs(/.*)?', '/sos_reports(/.*)?']
+        if any(re.match(skip, orig_path) for skip in skips):
+            return None
         try:
             fstat = os.stat(orig_path)
         except OSError:
             return tarinfo
         if self._with_selinux_context:
             context = self.get_selinux_context(orig_path)
-            if(context):
+            if context:
                 tarinfo.pax_headers['RHT.security.selinux'] = context
         self.set_tarinfo_from_stat(tarinfo, fstat)
         return tarinfo
@@ -676,51 +681,42 @@ class TarFileArchive(FileCacheArchive):
             return None
 
     def name(self):
-        return "%s.%s" % (self._name, self._suffix)
+        return "%s.%s" % (self._archive_root, self._suffix)
 
     def name_max(self):
         # GNU Tar format supports unlimited file name length. Just return
         # the limit of the underlying FileCacheArchive.
         return super(TarFileArchive, self).name_max()
 
-    def _build_archive(self):
-        tar = tarfile.open(self._archive_name, mode="w")
+    def _build_archive(self, method):
+        if method == 'auto':
+            method = 'xz' if find_spec('lzma') is not None else 'gzip'
+        _comp_mode = method.strip('ip')
+        self._archive_name = self._archive_name + ".%s" % _comp_mode
+        # tarfile does not currently have a consistent way to define comnpress
+        # level for both xz and gzip ('preset' for xz, 'compresslevel' for gz)
+        if method == 'gzip':
+            kwargs = {'compresslevel': 6}
+        else:
+            kwargs = {'preset': 3}
+        tar = tarfile.open(self._archive_name, mode="w:%s" % _comp_mode,
+                           **kwargs)
+        # add commonly reviewed files first, so that they can be more easily
+        # read from memory without needing to extract the whole archive
+        for _content in ['version.txt', 'sos_reports', 'sos_logs']:
+            if not os.path.exists(os.path.join(self._archive_root, _content)):
+                continue
+            tar.add(
+                os.path.join(self._archive_root, _content),
+                arcname=f"{self._name}/{_content}"
+            )
         # we need to pass the absolute path to the archive root but we
         # want the names used in the archive to be relative.
-        tar.add(self._archive_root, arcname=os.path.split(self._name)[1],
+        tar.add(self._archive_root, arcname=self._name,
                 filter=self.copy_permissions_filter)
         tar.close()
+        self._suffix += ".%s" % _comp_mode
+        return self.name()
 
-    def _compress(self):
-        methods = []
-        # Make sure that valid compression commands exist.
-        for method in ['xz', 'gzip']:
-            if is_executable(method):
-                methods.append(method)
-            else:
-                self.log_info("\"%s\" compression method unavailable" % method)
-        if self.method in methods:
-            methods = [self.method]
-
-        exp_msg = "No compression utilities found."
-        last_error = Exception(exp_msg)
-        for cmd in methods:
-            suffix = "." + cmd.replace('ip', '')
-            cmd = self._policy.get_cmd_for_compress_method(cmd, self._threads)
-            try:
-                exec_cmd = "%s %s" % (cmd, self.name())
-                r = sos_get_command_output(exec_cmd, stderr=True, timeout=0)
-
-                if r['status']:
-                    self.log_error(r['output'])
-                    raise Exception("%s exited with %s" % (exec_cmd,
-                                    r['status']))
-
-                self._suffix += suffix
-                return self.name()
-
-            except Exception as e:
-                last_error = e
-        raise last_error
 
 # vim: set et ts=4 sw=4 :
