@@ -20,17 +20,14 @@ class SoSUsernameMap(SoSMap):
     Note that this specifically obfuscates user_names_ and not UIDs.
     """
 
+    ignore_short_items = True
+    match_full_words_only = True
     name_count = 0
 
-    def load_names_from_options(self, opt_names):
-        for name in opt_names:
-            if name and name not in self.dataset.keys():
-                self.add(name)
-
     def sanitize_item(self, username):
         """Obfuscate a new username not currently found in the map
         """
-        ob_name = "obfuscateduser%s" % self.name_count
+        ob_name = f"obfuscateduser{self.name_count}"
         self.name_count += 1
         if ob_name in self.dataset.values():
             return self.sanitize_item(username.lower())
