@@ -13,7 +13,14 @@ from sos.collector.clusters import Cluster
 
 
 class satellite(Cluster):
-    """Red Hat Satellite 6"""
+    """
+    This profile is specifically for Red Hat Satellite 6, and not earlier
+    releases of Satellite.
+
+    While note technically a 'cluster' in the traditional sense, Satellite
+    does provide for 'capsule' nodes which is what this profile aims to
+    enumerate beyond the 'primary' Satellite system.
+    """
 
     cluster_name = 'Red Hat Satellite 6'
     packages = ('satellite', 'satellite-installer')
@@ -25,16 +32,18 @@ class satellite(Cluster):
 
     def get_nodes(self):
         cmd = self._psql_cmd('copy (select name from smart_proxies) to stdout')
-        res = self.exec_master_cmd(cmd, need_root=True)
+        res = self.exec_primary_cmd(cmd, need_root=True)
         if res['status'] == 0:
             nodes = [
-                n.strip() for n in res['stdout'].splitlines()
+                n.strip() for n in res['output'].splitlines()
                 if 'could not change directory' not in n
             ]
             return nodes
         return []
 
     def set_node_label(self, node):
-        if node.address == self.master.address:
+        if node.address == self.primary.address:
             return 'satellite'
         return 'capsule'
+
+# vim: set et ts=4 sw=4 :
