@@ -20,9 +20,7 @@ class Apt(Plugin, DebianPlugin, UbuntuPlugin):
 
     def setup(self):
         self.add_copy_spec([
-            "/etc/apt",
-            "/var/log/apt",
-            "/var/log/unattended-upgrades"
+            "/etc/apt", "/var/log/apt"
         ])
 
         self.add_forbidden_path("/etc/apt/auth.conf")
