@@ -6,10 +6,10 @@
 #
 # See the LICENSE file in the source distribution for further information.
 
-from sos.report.plugins import Plugin, IndependentPlugin
+from sos.report.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin
 
 
-class Gdm(Plugin, IndependentPlugin):
+class Gdm(Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin):
 
     short_desc = 'GNOME display manager'
 
