@@ -19,12 +19,18 @@ class Pam(Plugin):
     security_libs = ""
 
     def setup(self):
+
+        self.add_file_tags({
+            '/etc/pam.d/password-auth': 'password_auth',
+            '/etc/security/limits.*.conf': 'limits_conf'
+        })
+
         self.add_copy_spec([
             "/etc/pam.d",
             "/etc/security"
         ])
         self.add_cmd_output([
-            "ls -lanF %s" % self.security_libs,
+            f"ls -lanF {self.security_libs}",
             "pam_tally2",
             "faillock"
         ])
@@ -34,14 +40,12 @@ class RedHatPam(Pam, RedHatPlugin):
     security_libs = "/lib*/security"
 
     def setup(self):
-        super(RedHatPam, self).setup()
+        super().setup()
+        self.add_cmd_output(["authselect current"])
 
 
 class DebianPam(Pam, DebianPlugin, UbuntuPlugin):
     security_libs = "/lib/x86_64-linux-gnu/security"
 
-    def setup(self):
-        super(DebianPam, self).setup()
-
 
 # vim: set et ts=4 sw=4 :
