@@ -24,11 +24,8 @@ class Stratis(Plugin, RedHatPlugin):
             'pool list',
             'filesystem list',
             'blockdev list',
-            'key list',
             'daemon redundancy',
-            'daemon version',
-            'report engine_state_report',
-            '--version',
+            'daemon version'
         ]
 
         self.add_cmd_output(["stratis %s" % subcmd for subcmd in subcmds])
