@@ -71,6 +71,15 @@ def add_info(report, ui):
     # non-development release.
     apport.hookutils.attach_conffiles(report, 'apport', ui=ui)
 
+    casper_md5check = 'casper-md5check.json'
+    if 'LiveMediaBuild' in report:
+        apport.hookutils.attach_casper_md5check(report,
+                                                '/run/%s' % casper_md5check)
+    else:
+        apport.hookutils.attach_casper_md5check(report,
+                                                '/var/log/installer/%s' %
+                                                casper_md5check)
+
     if report.get('ProblemType') == 'Package':
         # every error report regarding a package should have package manager
         # version information
@@ -246,18 +255,6 @@ def check_attachment_for_errors(report, attachment):
                 # File these reports on the initramfs-tools package instead
                 report['SourcePackage'] = 'initramfs-tools'
 
-        if report['Package'] in ['emacs22', 'emacs23', 'emacs-snapshot', 'xemacs21']:
-            # emacs add-on packages trigger byte compilation, which might fail
-            # we are very interested in reading the compilation log to determine
-            # where to reassign this report to
-            regex = r'^!! Byte-compilation for x?emacs\S+ failed!'
-            if attachment in report and re.search(regex, log_file, re.MULTILINE):
-                for line in log_file.split('\n'):
-                    m = re.search(r'^!! and attach the file (\S+)', line)
-                    if m:
-                        path = m.group(1)
-                        apport.hookutils.attach_file_if_exists(report, path)
-
         if report['Package'].startswith('linux-image-') and attachment in report:
             # /etc/kernel/*.d failures from kernel package postinst
             m = re.search(r'^run-parts: (/etc/kernel/\S+\.d/\S+) exited with return code \d+', log_file, re.MULTILINE)
@@ -355,6 +352,20 @@ def add_release_info(report):
     # https://bugs.launchpad.net/bugs/364649
     media = '/var/log/installer/media-info'
     apport.hookutils.attach_file_if_exists(report, media, 'InstallationMedia')
+    # Preinstalled Raspberry Pi images include a build date breadcrumb
+    apport.hookutils.attach_file_if_exists(report, '/.disk/info',
+                                           'ImageMediaBuild')
+    if 'ImageMediaBuild' in report:
+        add_tag(report, '%s-image' % report['Architecture'])
+        try:
+            with open('/proc/device-tree/compatible', 'rb') as f:
+                is_a_pi = any(vendor == 'raspberrypi'
+                              for s in f.read().split(b'\0') if s
+                              for vendor, model in (s.decode('ascii').split(',', 1),))
+        except FileNotFoundError:
+            is_a_pi = False
+        if is_a_pi:
+            add_tag(report, 'raspi-image')
 
     # if we are running from a live system, add the build timestamp
     apport.hookutils.attach_file_if_exists(
@@ -389,11 +400,6 @@ def add_release_info(report):
     else:
         report['UpgradeStatus'] = 'No upgrade log present (probably fresh install)'
 
-    # check for system-image version on phablet builds
-    if apport.hookutils.command_available('system-image-cli'):
-        report['SystemImageInfo'] = '%s' % apport.hookutils.command_output(
-            ['system-image-cli', '-i'], stderr=None)
-
 
 def add_proposed_info(report):
     '''Tag if package comes from -proposed'''
