@@ -1,19 +1,13 @@
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-
-# You should have received a copy of the GNU General Public License along
-# with this program; if not, write to the Free Software Foundation, Inc.,
-# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
+# This file is part of the sos project: https://github.com/sosreport/sos
+#
+# This copyrighted material is made available to anyone wishing to use,
+# modify, copy, or redistribute it subject to the terms and conditions of
+# version 2 of the GNU General Public License.
+#
+# See the LICENSE file in the source distribution for further information.
 
 from sos.plugins import Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin
-from shutil import which
+from sos.utilities import is_executable
 
 
 class Lstopo(Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin):
@@ -22,13 +16,13 @@ class Lstopo(Plugin, RedHatPlugin, DebianPlugin, UbuntuPlugin):
 
     plugin_name = "lstopo"
     profiles = ("system", "hardware")
-    packages = ("hwloc-libs", )
+    packages = ("hwloc-libs", "libhwloc5")
 
     def setup(self):
         # binary depends on particular package, both require hwloc-libs one
         # hwloc-gui provides lstopo command
         # hwloc provides lstopo-no-graphics command
-        if which("lstopo"):
+        if is_executable("lstopo"):
             cmd = "lstopo"
         else:
             cmd = "lstopo-no-graphics"
